
##############################################################################
#
#author nbessi
#Thanks to Laurent Lauden for his code adaptation
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import osv, fields
import re
import time
try:
	import ldap
	import ldap.modlist
except :
	print 'python ldap not installed please install it in order to use this module'
import sys 

class LdapConnMApper(object):
	"""LdapConnMApper: push specific fields from the Terp Partner_contacts to the
		LDAP schema inetOrgPerson. Ldap bind options are stored in company.r"""
	def __init__(self):
		self.USER_DN = ''
		self.CONTACT_DN = ''
		self.LDAP_SERVER = ''
		self.PASS = ''
		self.connexion = ''
	
	def init_connexion(self,cr,uid,context,osv):
		#getting ldap pref
		user = osv.pool.get('res.users').browse(cr, uid, uid, context=context)
		company = osv.pool.get('res.company').browse(cr, uid,user.company_id.id, context=context)
		self.USER_DN = company.base_dn
		self.CONTACT_DN = company.contact_dn
		self.LDAP_SERVER = company.ldap_server
		self.PASS = company.passwd
		if self.USER_DN == '' or self.CONTACT_DN == '' or self.LDAP_SERVER == '' or self.PASS == '' :
			raise osv.except_osv('Warning !', 'An LDAP parameter is missing for company %'%(company.name))
	
	def get_connexion(self):
		print 'connecting to server ldap '+ self.LDAP_SERVER
		try:
			self.connexion = ldap.open(self.LDAP_SERVER)
			self.connexion.simple_bind_s(self.USER_DN,self.PASS)
		except Exception, e:
			raise e
		return self.connexion
		
class contact_to_ldap_addressLdap(osv.osv):
	name = "contact_to_ldap.addressLdap"
	_inherit = 'res.partner.address'
	def create(self, cr, uid, vals, context={}):
		ids = None
		self.validate_entries(vals,cr,uid,ids)

		tmp_id = super(contact_to_ldap_addressLdap,self).create(cr, uid, vals, context)
		if self.ldaplinkactive(cr,uid,context):
			self.saveLdapContact(tmp_id,vals,cr,uid)
		return tmp_id
	
	def write(self, cr, uid, ids, vals, context={}) :
		if isinstance(ids, (int, long)):
			ids = [ids]
		self.validate_entries(vals,cr,uid,ids)
		succes = super(contact_to_ldap_addressLdap,self).write(cr, uid, ids, vals, context)
		if self.ldaplinkactive(cr,uid,context):
			for id in ids:
				self.updateLdapContact(id,vals,cr,uid)
		return succes

	def unlink(self, cr, uid, ids,context={}):
		if isinstance(ids, (int, long)):
			ids = [ids]
		if self.ldaplinkactive(cr,uid,context):
			for id in ids:
				self.removeLdapContact(id,cr,uid)
		return super(contact_to_ldap_addressLdap,self).unlink(cr, uid, ids)
	
	def validate_entries(self,vals,cr,uid,ids):
		for val in vals :
			try :
				if isinstance(vals[val], basestring):
					vals[val] = vals[val].encode('utf-8')
			except UnicodeError:
				pass
		if ids != None:
			if isinstance(ids, (int, long)):
				ids = [ids]
			if len(ids) == 1:
				self.addNeededFields(ids[0],vals,cr,uid)

		keys = vals.keys()
			
		email = False
		if 'email' in keys:
			email = vals['email']
			
		phone = False
		if 'phone' in keys:
			phone = vals['phone']
			
		fax=False
		if 'fax' in keys:
			fax = vals['fax']

		mobile = False
		if 'mobile' in keys:
			mobile = vals['mobile']

		lastname = False
		if 'lastname' in keys:
			lastname = vals['lastname']
			
		private_phone= False
		if 'private_phone' in keys:
			private_phone = vals['private_phone']
			
		
		# if name == False :
		# 	raise osv.except_osv('Warning !', 'Please enter a contact name')
		if email :
			if re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", email) == None :
				raise osv.except_osv('Warning !', 'Please enter a valid e-mail')
		if phone :
			if phone.startswith('+') == False :
				raise osv.except_osv('Warning !', 'Please enter a valid phone number in international format (i.e. leading +)')
		if fax :
			if fax.startswith('+') == False :
				raise osv.except_osv('Warning !', 'Please enter a valid fax number in international format (i.e. leading +)')
		if mobile :
			if mobile.startswith('+') == False :
				raise osv.except_osv('Warning !', 'Please enter a valid mobile number in international format (i.e. leading +)')
		if private_phone :
			if private_phone.startswith('+') == False :
				raise osv.except_osv('Warning !', 'Please enter a valid private phone number in international format (i.e. leading +)')
		if lastname == False and (ids == None or 'lastname' in keys):
			raise osv.except_osv('Warning !', 'Please enter at least a lastname in the address (i.e. leading +)')
	
	def getVals(self,att_name,key,vals,dico,uid,ids,cr) :
		if key in vals.keys() and vals[key] <> False :
			dico[att_name] = vals[key]
		else :
			tmp = self.read(cr, uid, ids, [key], context={})
			print tmp
			if tmp[key] :
				dico[att_name] = tmp[key]

	def addNeededFields(self,id,vals,cr,uid):	
		keys = vals.keys()

		previousvalue = self.browse(cr,uid,[id])[0]		
		
		
		if not 'partner_id' in keys :
			vals['partner_id'] = previousvalue.partner_id.id
			
		if not 'email' in keys:
			vals['email'] = previousvalue.email
			
		if not 'phone' in keys:
			vals['phone'] = previousvalue.phone
			
		if not 'fax' in keys:
			vals['fax'] = previousvalue.fax

		if not 'mobile' in keys:
			vals['mobile'] = previousvalue.mobile
			
		if not 'firstname' in keys:
			vals['firstname'] = previousvalue.firstname

		if not 'lastname' in keys:
			vals['lastname'] = previousvalue.lastname
			
		if not 'private_phone' in keys:
			vals['private_phone'] = previousvalue.private_phone
			
		if not 'street' in keys:
			vals['street'] = previousvalue.street
			
		if not 'street2' in keys:
			vals['street2'] = previousvalue.street2

		
	def mappLdapObject(self,id,vals,cr,uid):
		self.addNeededFields(id,vals,cr,uid)
		keys = vals.keys()
		partner_obj=self.pool.get('res.partner')
		part_name = partner_obj.browse(cr,uid,vals['partner_id']).name
		vals['partner'] = part_name
		cn = ''
		
		if 'lastname' in keys and 'firstname' in keys and vals['firstname'] <> False and vals['lastname'] <> False:
			cn = vals['lastname'] +' '+ vals['firstname']
		elif (('lastname' not in keys) or ('lastname' in keys and vals['lastname'] == False)) and 'firstname' in keys and vals['firstname'] <> False:
			cn = part_name + ' ' + vals['firstname']
		elif 'lastname' in keys and vals['lastname'] <> False and (('firstname' not in keys) or ('firstname' in keys and vals['firstname'] == False)) :
			cn = vals['lastname']
		else:
			cn = part_name
					
		contact_obj = {
		'objectclass' : ['organizationalPerson', 'inetOrgPerson'],
		'uid': ['terp_'+str(id)],
		'ou':['Contacts'],
		'cn':[cn],
		'sn':[vals['lastname']]
		}
		if not vals['street']:
			vals['street'] = ''

		if not vals['street2']:
			vals['street2'] = ''

		contact_obj['street'] = vals['street'] + ' ' + vals['street2']
			
		self.getVals('givenName', 'firstname',vals, contact_obj, uid, id, cr)
		self.getVals('mail', 'email',vals, contact_obj, uid, id, cr)
		self.getVals('o','partner' ,vals, contact_obj, uid, id, cr)
		self.getVals('telephoneNumber', 'phone',vals, contact_obj, uid, id, cr)
		self.getVals('l', 'city',vals, contact_obj, uid, id, cr)
		self.getVals('facsimileTelephoneNumber', 'fax',vals, contact_obj, uid, id, cr)
		self.getVals('mobile', 'mobile',vals, contact_obj, uid, id, cr)
		self.getVals('homePhone', 'private_phone',vals, contact_obj, uid, id, cr)
		self.getVals('postalCode', 'zip',vals, contact_obj, uid, id, cr)
		return contact_obj
	
	def saveLdapContact(self,id,vals,cr,uid):
		contact_obj = self.mappLdapObject(id,vals,cr,uid)
		print contact_obj
		conn = self.connectToLdap(cr,uid,context={})
		try:
			conn.connexion.add_s('uid=terp_'+str(id)+','+conn.CONTACT_DN, ldap.modlist.addModlist(contact_obj))
		except Exception, e:
			raise e
		conn.connexion.unbind_s()
		
	def updateLdapContact(self,id,vals,cr,uid):
		conn = self.connectToLdap(cr,uid,context={})
		try:
			old_contatc_obj = self.getLdapContact(conn,id)
		except ldap.NO_SUCH_OBJECT:
			self.saveLdapContact(id,vals,cr,uid)
			return
		contact_obj = self.mappLdapObject(id,vals,cr,uid)
		modlist = ldap.modlist.modifyModlist(old_contatc_obj[1], contact_obj)
		try:
			conn.connexion.modify_s(old_contatc_obj[0], modlist)
			conn.connexion.unbind_s()
		except Exception, e:
			raise e
			
	def removeLdapContact(self,id,cr,uid):
		conn = self.connectToLdap(cr,uid,context={})
		to_delete = None
		try:
			#print 'removing object in ldap'
			to_delete = self.getLdapContact(conn,id)
			#print 'removing object in ldap done'
		except ldap.NO_SUCH_OBJECT:
			print 'no object to delete in ldap'
		except Exception, e :
			raise e
		try:
			if to_delete :
				conn.connexion.delete_s(to_delete[0])
				conn.connexion.unbind_s()
		except Exception, e:
			raise e
		
	def getLdapContact(self, conn, id):
		print conn
		result = conn.connexion.search_ext_s(conn.CONTACT_DN,ldap.SCOPE_SUBTREE,"(&(objectclass=*)(uid=terp_"+str(id)+"))")
		if len(result) == 0:
			raise ldap.NO_SUCH_OBJECT
		return result[0]
		
	def ldaplinkactive(self,cr,uid,context):
		user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
		company = self.pool.get('res.company').browse(cr, uid,user.company_id.id, context=context)
		return company.ldap_active
		
	def connectToLdap(self,cr,uid,context):
		#getting ldap pref
		ldapMapper = LdapConnMApper()
		ldapMapper.init_connexion(cr,uid,context,self)
		ldapMapper.get_connexion()
		return ldapMapper

			
class ldap_partner(osv.osv):
	name = "contact_to_ldap.ldap_partner"
	_inherit = ('res.partner')
	
	def unlink(self, cr, uid, ids):
		obj = self.pool.get('res.partner.address')
		if isinstance(ids, (int, long)):
			ids = [ids]
		obj_ids = obj.search(cr,uid, [('partner_id','in',ids)])
		for address in obj.browse(cr,uid,obj_ids):
			address.unlink(cr,uid,address.id)
		return super(ldap_partner,self).unlink(cr, uid, ids)
contact_to_ldap_addressLdap()
ldap_partner()