# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 Camptocamp SA
# @author JB Aubort
# @source Network Gulf IT's currency update module for
#         NYFE and ECB sources and their ideas
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import osv, fields
import time
from mx import DateTime
import netsvc
import string

# ------------------------------------
#              Rate
# ------------------------------------
class currency_rate_update(osv.osv):

    logger = netsvc.Logger()
    LOG_NAME = 'cron-rates'
    MOD_NAME = 'c2c_currency_rate_update: '

# ------------------------------------
#  Get Default Company
# ------------------------------------
    def _get_def_company(self, cr, uid, context):
        return self.pool.get('res.company').search(cr, uid, [])[0]

# ------------------------------------
#  Get Default Currency
# ------------------------------------
    def _get_def_currency(self, cr, uid, ids, name={}, arg={}, context={}):
        company_id = self._get_def_company(cr, uid, context)
        return self.pool.get('res.company').browse(cr, uid, [company_id])[0].currency_id.id

# ------------------------------------
#         Create rate
# ------------------------------------
    def create(self, cr, uid, vals, context):
        rate_id = super(currency_rate_update,self).create(cr, uid, vals, context)
        if rate_id and self._table=='currency_rate_update':
            rate_id = int(rate_id)
            cron = self._start_cron(cr, uid, [rate_id])
        return rate_id

# ------------------------------------
#         Write rate
# ------------------------------------
    def write(self, cr, uid, ids, vals, context={}):
        succes = super(currency_rate_update,self).write(cr, uid, ids, vals, context)
        if succes and self._table=='currency_rate_update':
            if type(ids)==type([]):
                rate_id = int(ids[0])
            else:
                rate_id = int(ids)
            cron = self._start_cron(cr, uid, [rate_id])
        return succes

# ------------------------------------
#         Delete rate
# ------------------------------------
    def unlink(self, cr, uid, rate_id, context={}):
        rate = self.pool.get('currency.rate.update').browse(cr, uid, rate_id)[0]
        cron_id = rate.cron_id.id
        succes = super(currency_rate_update,self).unlink(cr, uid, rate_id, context)
        if succes:
            self._del_cron(cr, uid, [cron_id])
        return succes

# ------------------------------------
#          Start cron
# ------------------------------------
    def _start_cron(self, cr, uid, ids):
        rate = self.pool.get('currency.rate.update').browse(cr, uid, ids)[0]
        rate_id = int(rate.id)
        if rate.active:
            if rate.cron_id:
                cron_id = rate.cron_id.id
                self.pool.get('ir.cron').write(cr, uid, [cron_id],{
                    'interval_type': rate.interval_type,
                    'interval_number': rate.interval_time,
                    'active': True,
                    'nextcall': rate.nextcall,
                })
            else:
                res = {
                    'name': 'Currency Rate Update #%i'%rate_id,
                    'model': 'currency.rate.update',
                    'user_id': uid,
                    'priority': 1,
                    'numbercall': -1,
                    'doall': False,
                    'nextcall': rate.nextcall,
                    'interval_type': rate.interval_type,
                    'interval_number': rate.interval_time,
                    'function': 'update',
                    'args':  (rate_id,),
                    'active': True,
                }
                cron_id = self.pool.get('ir.cron').create(cr, uid, res)
                self.write(cr, uid, rate_id, {'cron_id':cron_id})
            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_INFO, 'cron #%i active'%cron_id)
        elif rate.cron_id.id:
            self._stop_cron(cr, uid, rate.cron_id.id)
        return True

# ------------------------------------
#          Stop cron
# ------------------------------------
    def _stop_cron(self, cr, uid, cron_id):
        succes = self.pool.get('ir.cron').write(cr, uid, [cron_id], {'active': False})
        if succes:
            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_INFO, 'cron #%i disabled'%int(cron_id))
        return succes

# ------------------------------------
#          Delete cron
# ------------------------------------
    def _del_cron(self, cr, uid, cron_id):
        succes = self.pool.get('ir.cron').unlink(cr, uid, cron_id)
        if succes:
            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_INFO, 'cron #%i deleted'%int(cron_id[0]))
        return succes

# ------------------------------------
#   Get rate (from cron)
# ------------------------------------
    def update(self, cr, uid, rate_id):
        rate = self.pool.get('currency.rate.update').browse(cr, uid, [rate_id])[0]
        service = rate.service
        company_id = rate.company_id.id
        ref_to = rate.ref_currency.code
        rates = self.process_source(service)
        rates = self.calc_rates_for_ref(ref_to, rates)
        current_date = time.strftime('%Y-%m-%d')
        if rates:
            for rate in rates:
                currency_id = self.pool.get('res.currency').search(cr, uid, [('code', '=', rate[0].upper())])
                if currency_id:
                    rates_for_currency = self.pool.get('res.currency.rate').search(cr, uid, [
                        ('currency_id', '=', currency_id[0])
                    ])
                    rates_for_currency.sort()
                    
                    if rates_for_currency:
                        current_rate_id = rates_for_currency[len(rates_for_currency)-1]
                        current_rate_obj = self.pool.get('res.currency.rate').browse(cr, uid, [current_rate_id])[0]
                        current_rate_name = current_rate_obj.name
                        current_rate = current_rate_obj.rate
                    else:
                        current_rate = -1
                        current_rate_name = ''
                    if rate[1]!=0 and not self.equal(current_rate, rate[1], 6):
                        res_cur_obj = self.pool.get('res.currency.rate')
                        if current_rate_name==current_date:
                            res_cur_obj.write(cr,uid,[current_rate_id],{'rate':rate[1]})
                        else:
                            res_cur_obj.create(cr,uid,{
                                'currency_id': currency_id[0],
                                'rate':rate[1],
                                'name': current_date
                            })
                        updated = True
                    else:
                        updated = False
                    # User CR because self.write is overloaded and it'll not work
                    cr.execute('update currency_rate_update set lastcall=%s where id=%i',(time.strftime('%Y-%m-%d %H:%M:%S'), rate_id))
                    cr.commit()
        if updated:
            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_INFO, 'rate #%i updated'%int(rate_id))
            return True
        else:
            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_INFO, 'rate #%i already up-to-date'%int(rate_id))
            return False

# ------------------------------------
#         WS Sources
# ------------------------------------
    def process_source(self, service):
        
        if service=='admin-ch':
            url = 'http://www.afd.admin.ch/publicdb/newdb/mwst_kurse/wechselkurse.php'
        elif service=='boc':
            url = 'http://www.bankofcanada.ca/fr/marches/csv/exchange_fre.csv'
        elif service=='google':
            url = 'http://finance.google.com/finance?q='
        elif service=='ecb':
            url = 'http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml'
        elif service=='nyfb':
            url = 'http://www.newyorkfed.org/markets/fxrates/FXtoXML.cfm?FEXdate=%s&FEXtime=1200'
        else:
            url = ''
        
        try:
            import urllib
            objfile = urllib.urlopen(url)
            rawfile = objfile.read()
            objfile.close()
        except ImportError:
            raise osv.except_osv('Error !', self.MOD_NAME+'Unable to import urllib !')
        except IOError:
            raise osv.except_osv('Error !', self.MOD_NAME+'Web Service does not exist !')
        
        rates =[]
        
        # ---------------------
        # Admin.ch
        if service=='admin-ch':
            try:
                from xml.dom.minidom import parseString
            except ImportError:
                raise osv.except_osv('Error !', self.MOD_NAME+'Unable to import xml.dom.minidom !')
            
            try:
                dom = parseString(rawfile)
            except: # Wich exception ?
                raise osv.except_osv('Error !', self.MOD_NAME+'XML not well-formed !')
            
            date = dom.getElementsByTagName('datum')[0].childNodes[0].data
            hour = dom.getElementsByTagName('zeit')[0].childNodes[0].data
            for node in dom.getElementsByTagName('devise'):
                code = node.attributes['code'].value.lower()
                currency = node.getElementsByTagName('waehrung')[0].childNodes[0].data
                rate_currency = float(node.getElementsByTagName('kurs')[0].childNodes[0].data)
                rate_ref = float(currency.lower().replace(code, '').strip())
                #Source is in CHF, so we transform it into reference currencies
                if rate_currency!=0:
                    rate = 1 / (rate_currency / rate_ref)
                else:
                    rate = 0
                rates.append([code, rate])
            rates.append([u'chf', 1.0])
            return ['CHF', rates]
        
        # ---------------------
        # Bank of Canada
        elif service=='boc':
            try:
                import os
                import re
            except ImportError:
                raise osv.except_osv('Error !', self.MOD_NAME+'Unable to import "re" or "os" module !')
            
            for line in rawfile.split( os.linesep ):
                if len(line)>0 and line[0]!='#':
                    line = re.sub("(.*,)*", "", line)
                    rates.append(line)
            rates.pop(0)
            rates[0] = '1.000'
            
            # Mapping because currency codes are not in the CSV source
            mapping = [
                'cad','usd','ars','aud','bsd','brl','clp',
                'cny','cop','hrk','czk','dkk','xcd','eur',
                'fjd','cfa','cfp','ghc','gtq','hnl','hkd',
                'huf','isk','imr','idr','nis','jmd','jpy',
                'myr','mxn','mad','mkk','ang','nzd','nok',
                'pkr','pab','pen','php','pln','ron','rub',
                'rsd','sgd','skk','zar','krw','lkr','sek',
                'chf','twd','thb','ttd','tnd','try','aed',
                'gbp','vef','vnd']
            
            for i in range(len(rates)):
                #Source is in CAD (Canadian Dollar), so we transform it into reference currencies
                rates[i] = [mapping[i], float(1) / float(rates[i][:-2])]
            
            return ['CAD', rates]
        
        # ---------------------
        # Google Finance
        elif service=='google':
            import urllib
            import re
            # Currencies supported by Google (taken from the select field in finance.google.com)
            currencies = [
                'AED','ANG','ARS','AUD','BGN','BHD','BND','BOB','BRL','BWP',
                'CAD','CHF','CLP','CNY','COP','CSD','CZK','DKK','EEK','EGP',
                'EUR','FJD','GBP','HKD','HNL','HRK','HUF','IDR','ILS','INR',
                'ISK','JPY','KRW','KWD','KZT','LKR','LTL','MAD','MUR','MXN',
                'MYR','NOK','NPR','NZD','OMR','PEN','PHP','PKR','PLN','QAR',
                'RON','RUB','SAR','SEK','SGD','SIT','SKK','THB','TRY','TTD',
                'TWD','UAH','USD','VEB','ZAR']
            
            for currency in currencies:
                url_cur = '%sEUR%s'%(url, currency)
                content = urllib.urlopen(url_cur).read()
                search = re.search('=<span class=bld>&nbsp;(.*?)&nbsp;(.*?)&nbsp;<', content)
                if search:
                    quote = float(search.group(1).replace(',', ''))
                    rates.append([currency.lower(), quote])
            rates.append([u'eur', 1.0])
            return ['EUR', rates]
        
        # ---------------------
        # Federal Reserve Bank of New York
        elif service=='nyfb':
            try:
                import urllib
                import datetime
                from xml.dom import minidom
            except ImportError:
                raise osv.except_osv('Error !', self.MOD_NAME+'Unable to import some libraries !')
            
            date = (datetime.datetime.now() - datetime.timedelta(days=1)).strftime('%Y-%m-%d')
            server_url = url % date
            xmldoc = minidom.parse(urllib.urlopen(server_url))
            for child in xmldoc.getElementsByTagName('frbny:Series'):
                currency = str(child.getAttribute('UNIT')).lower()
                if currency != 'usd':
                    quote = float(child.childNodes[1].childNodes[1].firstChild.toxml())
                    rates.append([currency, quote])
            rates.append([u'usd', 1.0])
            
            return ['USD', rates]
        
        # ---------------------
        # Europe Central Bank
        elif service=='ecb':
            try:
                from xml.dom.minidom import parseString
                dom = parseString(rawfile)
            except ImportError:
                raise osv.except_osv('Error !', self.MOD_NAME+'Unable to import xml.dom.minidom !')
            except: # Wich exception ?
                raise osv.except_osv('Error !', self.MOD_NAME+'XML not well-formed !')
            
            for child in dom.getElementsByTagName('Cube'):
                if child.hasAttribute("currency") and child.hasAttribute("rate"):
                    currency = str(child.getAttribute("currency")).lower()
                    quote = float(child.getAttribute("rate"))
                    rates.append([currency, quote])
            rates.append([u'eur', 1.0])
            return ['EUR', rates]
        
        else:
            ##TODO: Yahoo ?
            return ['EUR', [[u'eur',1.0]]]

# ------------------------------------
#      Rates calculation
# ------------------------------------
    def calc_rates_for_ref(self, ref_to, infos):
        ref_from = infos[0].lower()
        ref_to = unicode(ref_to.lower())
        rates = infos[1]
        checked = False
        for x in rates:
            if ref_to in x:
                checked = True
                cur_to = float(x[1])
                for i in range(len(rates)):
                    rates[i][1] = rates[i][1] / cur_to
                break
        if not checked:
            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_WARNING, '%s does not exist in source !'%ref_to)
            raise osv.except_osv('Error !', self.MOD_NAME+'Destination currency does not exist in exchange rates source !')
        elif rates:
            return rates
        else:
            raise osv.except_osv('Error !', self.MOD_NAME+'Something went wrong in calc_rates_for_ref() !')

# ------------------------------------
#            Tools
# ------------------------------------

    def equal(self, num1, num2, accuracy):
        return abs(num1 - num2) < 10**(-accuracy)

# ------------------------------------
#         Definitions
# ------------------------------------
    _name = "currency.rate.update"
    _description = "Currency Rate Update"
    _columns = {
        'service': fields.selection([
            ('admin-ch','Admin.ch'),
            ('ecb','European Central Bank'),
            ('nyfb','Federal Reserve Bank of New York'),
            ('google','Google Finance'),
            ('boc','Bank of Canada')
        ], 'Services', required=True),
        'ref_currency': fields.many2one('res.currency', 'Reference Currency', required=True),
        'nextcall' : fields.datetime('First call date', required=True),
        'lastcall' : fields.datetime('Last call date'),
        'interval_time': fields.integer('Interval', required=True),
        'interval_type': fields.selection([
            ('months','Months'),
            ('weeks','Weeks'),
            ('work_days','Work Days'),
            ('days','Days'),
        ], 'Interval Type', required=True),
        'active': fields.boolean('Active'),
        'cron_id': fields.many2one('ir.cron', 'Cron Job'),
        'company_id': fields.many2one('res.company', 'Company', required=True),
        'comment': fields.text('Comment'),
    }
    _defaults = {
        'nextcall' : lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),
        'interval_time': lambda *a: 1,
        'interval_type': lambda *a: 'work_days',
        'active': lambda *a: True,
        'cron_id': lambda *a: False,
        'ref_currency': _get_def_currency,
        'company_id': _get_def_company,
    }

currency_rate_update()
