# -*- coding: utf-8 -*- 
##############################################################################
#
# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
#                    Fabien Pinckaers <fp@tiny.Be>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from osv import osv, fields
import time
from mx import DateTime

import netsvc


#--------------------------------------------------------------------------------------------------------
# Modifiy line expense for readonly product when expense confirm
#--------------------------------------------------------------------------------------------------------
class hr_expense_line (osv.osv):
	_name = 'hr.expense.line'
	_inherit = "hr.expense.line"
	_columns  = {
		'product_id': fields.many2one('product.product', 'Product', readonly=True, states={'draft':[('readonly',False)]}, required=True),
		'uom_id': fields.many2one('product.uom', 'UoM', readonly=True),
		
	}
	
	def onchange_product_id(self, cr, uid, ids, product_id, uom_id, context={}):
		v={}
		if product_id:
			product=self.pool.get('product.product').browse(cr,uid,product_id, context=context)
			# v['name']=product.name
			v['unit_amount']=product.standard_price
			if not uom_id:
				v['uom_id']=product.uom_id.id
		return {'value':v}
		
		
hr_expense_line()
#--------------------------------------------------------------------------------------------------------
# Modifiy expense to readdonly field when confirmed
#--------------------------------------------------------------------------------------------------------

class hr_expense_expense(osv.osv):
	_name = "hr.expense.expense"
	_inherit = "hr.expense.expense"


	def action_invoice_create(self, cr, uid, ids):
		res = False
		for exp in self.browse(cr, uid, ids):
			
			lines = []
			for l in exp.line_ids:
				tax_id = []
				if l.product_id:
					acc = l.product_id.product_tmpl_id.property_account_expense.id
					if not acc:
						acc = l.product_id.categ_id.property_account_expense_categ.id
					tax_id = [x.id for x in l.product_id.supplier_taxes_id]
				else:
					acc = self.pool.get('ir.property').get(cr, uid, 'property_account_expense_categ', 'product.category')
				lines.append((0, False, {
					'name': str(l.date_value) + ' - ' + l.name ,
					'account_id': acc,
					'price_unit': l.unit_amount,
					'quantity': l.unit_quantity,
					'uos_id': l.uom_id.id,
					'product_id': l.product_id and l.product_id.id or False,
					'invoice_line_tax_id': tax_id and [(6, 0, tax_id)] or False,
					'account_analytic_id': l.analytic_account.id,
				}))
			if not exp.employee_id.address_id:
				raise osv.except_osv('Error !', 'The employee must have a contact address')
			acc = exp.employee_id.address_id.partner_id.property_account_payable.id
			bank_id = False
			if exp.employee_id.address_id.partner_id:
				if exp.employee_id.address_id.partner_id.bank_ids:
					bank_id = exp.employee_id.address_id.partner_id.bank_ids[0].id
			
			inv = {
				'name': exp.name,
				'reference': self.pool.get('ir.sequence').get(cr, uid, 'hr.expense.invoice'),
				'account_id': acc,
				'type': 'in_invoice',
				'partner_id': exp.employee_id.address_id.partner_id.id,
				'address_invoice_id': exp.employee_id.address_id.id,
				'address_contact_id': exp.employee_id.address_id.id,
				'origin': exp.name,
				'invoice_line': lines,
				'price_type': 'tax_included',
				'currency_id': exp.currency_id.id,
				'partner_bank':bank_id,
			}
			if exp.journal_id:
				inv['journal_id']=exp.journal_id.id
			inv_id = self.pool.get('account.invoice').create(cr, uid, inv, {'type':'in_invoice'})
			self.pool.get('account.invoice').button_compute(cr, uid, [inv_id], {'type':'in_invoice'}, set_total=True)
			self.write(cr, uid, [exp.id], {'invoice_id': inv_id, 'state': 'invoiced'})
			res = inv_id
			return res
		
		
	# set to draft
	def action_to_draft(self, cr, uid, ids, *args):
		self.write(cr, uid, ids, {'state':'draft'})
		wf_service = netsvc.LocalService("workflow")
		for inv_id in ids:
			wf_service.trg_create(uid, 'hr.expense.expense', inv_id, cr)
		return True
		
		
	_columns = {
	'name': fields.char('Expense sheet object', size=128, required=True, readonly=True, states={'draft':[('readonly',False)]}),
	'id': fields.integer('Expense sheet number', readonly=True),
	'ref': fields.char('Project number', size=32),
	'date': fields.date('Date', readonly=True, states={'draft':[('readonly',False)]}),
	'employee_id': fields.many2one('hr.employee', 'Employee', required=True, readonly=True, states={'draft':[('readonly',False)]}),
	'line_ids': fields.one2many('hr.expense.line', 'expense_id', 'Expense Lines', readonly=True, states={'draft':[('readonly',False)]}),
	'currency_id': fields.many2one('res.currency', 'Currency', required=True, readonly=True, states={'draft':[('readonly',False)]}),
	}
hr_expense_expense()

