#
#  account_move_line.py
#  c2c_fiscal_year_close
#
#  Created by Nicolas Bessi on 28.01.2008.
#  Copyright (c) 2008 CamptoCamp. All rights reserved.
#


from osv import osv, fields
import pooler
import time
import netsvc


class account_move_line(osv.osv):
	"""Class that inherits account_move_line that add a new reconcile 
		method that is less restrictive """

	_inherit = "account.move.line"
	_columns = {

		## link to the origini unreconciles entries
		"origin_link" : fields.many2one('account.move.line', 
			'Parent line', select=True),
	}
	
	## @param self The object pointer.
	## @param cr a psycopg cursor
	## @param uid res.user.id that is currently loged
	## @param type auto or manual (auto is used for wizard)s
	## @parma writeoff_acc_id id of write off account
	## @parma writeoff_period_id id of write off period
	## @parma writeoff_journal_id id of write off journal
	## @parma context a context dict
	## @return id of the created reconciliation or False
	def easy_reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False,
	 	writeoff_period_id=False, writeoff_journal_id=False, context={}):
		""" Reconciliation Method that return a complete reconciliation id"""
		id_set = ','.join(map(str, ids))
		lines = self.read(cr, uid, ids, context=context)
		unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
		credit = debit = 0.0
		currency = 0.0
		account_id = False
		partner_id = False
		for line in unrec_lines:
			if line['state'] <> 'valid':
				raise osv.except_osv('Error',
						'Entry "%s" is not valid !' % line.name)
			credit += line['credit']
			debit += line['debit']
			currency += line['amount_currency'] or 0.0
			account_id = line['account_id'][0]
			partner_id = (line['partner_id'] and line['partner_id'][0]) or False
		writeoff = debit - credit
		
		# Ifdate_p in context => take this date
		if context.has_key('date_p') and context['date_p']:
			date=context['date_p']
		else:
			date = time.strftime('%Y-%m-%d')

		cr.execute('SELECT account_id, reconcile_id \
				FROM account_move_line \
				WHERE id IN ('+id_set+') \
				GROUP BY account_id,reconcile_id')
		r = cr.fetchall()
		account = self.pool.get('account.account').browse(cr, uid, account_id, context=context)
		if (not self.pool.get('res.currency').is_zero(cr, uid, account.company_id.currency_id, writeoff)) or \
		   (account.currency_id and (not self.pool.get('res.currency').is_zero(cr, uid, account.currency_id, currency))):
			if not writeoff_acc_id:
				raise osv.except_osv('Warning', 
					'You have to provide an account for the write off entry !')
			if writeoff > 0:
				debit = writeoff
				credit = 0.0
				self_credit = writeoff
				self_debit = 0.0
			else:
				debit = 0.0
				credit = -writeoff
				self_credit = 0.0
				self_debit = -writeoff
			# If comment exist in context, take it
			if context.has_key('comment') and context['comment']:
				libelle=context['comment']
			else:
				libelle='Write-Off'
			
			writeoff_lines = [
				(0, 0, {
					'name':libelle,
					'debit':self_debit,
					'credit':self_credit,
					'account_id':account_id,
					'date':date,
					'partner_id':partner_id,
					'currency_id': account.currency_id.id or False,
					'amount_currency': account.currency_id.id and -currency or 0.0
				}),
				(0, 0, {
					'name':libelle,
					'debit':debit,
					'credit':credit,
					'account_id':writeoff_acc_id,
					'date':date,
					'partner_id':partner_id
				})
			]

			name = 'Write-Off'
			if writeoff_journal_id:
				journal = self.pool.get('account.journal').browse(
					cr, uid, writeoff_journal_id)
				if journal.sequence_id:
					name = self.pool.get('ir.sequence').get_id(
						cr, uid, journal.sequence_id.id)

			writeoff_move_id = self.pool.get('account.move').create(
				cr, 
				uid, 
				{
					'name': name,
					'period_id': writeoff_period_id,
					'journal_id': writeoff_journal_id,
					'state': 'draft',
					'line_id': writeoff_lines
					}
			)

			writeoff_line_ids = self.search(cr, uid, [('move_id', '=', writeoff_move_id), 
				('account_id', '=', account_id)])
			ids += writeoff_line_ids
			
		r_id = self.pool.get('account.move.reconcile').create(
			cr, 
			uid, {
				'type': type, 
				'line_id': map(lambda x: (4,x,False), ids)
		})
		# the id of the move.reconcile is written in the move.line (self) by the create method above 
		# because of the way the line_id are defined: (4, x, False)
		wf_service = netsvc.LocalService("workflow")
		for id in ids:
			wf_service.trg_trigger(uid, 'account.move.line', id, cr)
		return r_id
		
account_move_line()

class shellaccount(osv.osv):
	"""Shell class used for creating a view that will match the wrong closing method"""
	_name='c2c_fiscal_year_close.shellaccount'
	_auto = False
	_log_access = False
	_order = 'code,name,typename'
	
	_columns = {
		## account name
		'name':fields.char('name', size=500),
		
		## account type name
		'acctypename':fields.char('account type name', size=500),
		
		## account type name
		'acctype':fields.char('account type close_method', size=500),
		
		## account type name
		'typename':fields.char('account.account.type close_method', size=500),
		
		## account code
		'code':fields.char('code', size=500)
	}
	
	## @param self The object pointer.
	## @param cr a psycopg cursor
	def init(self, cr):
		"""This will create or update the view c2c_fiscal_year_close_shellaccount when the module is installed or updated"""
		
		cr.execute("""create or replace view c2c_fiscal_year_close_shellaccount as (\
			SELECT account_account.id, account_account.code, account_account.name, \
			account_account.type as acctypename, account_account.close_method as acctype, \
			account_account_type.close_method as typename \
			from account_account, account_account_type where \
			account_account.type = account_account_type.code and \
			account_account.close_method != account_account_type.close_method \
			and account_account.type !='view');"""
		)
shellaccount()