# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) Camptocamp SA
# Author: Arnaud Wüst
#
#
#    This file is part of the c2c_holidays_report module
#
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from c2c_reporting_tools.reports.standard_report import StandardReport
from c2c_reporting_tools.translation import _
from c2c_reporting_tools.flowables.timebox_chart import  *



class Department :
    
    display = True
    
    def __init__(self, browseRec):
        'Constructor that take a department browse record'
        self.br = browseRec
        self.name = self.br.name
        self.id = self.br.id 
        self.employees = [] 
        
    def __cmp__(self, other):
        'comparator function'
        return cmp(self.name, other.name) 
    
class NoDepartment(Department):
    """ department to regroup users who do not have departement """
    
    def __init__(self):
        self.br = None
        self.name = 'No Department'
        self.id = None
        self.employees = [] 
        
    def __cmp__(self, other):
        """always bigger than others"""
        if self.name == other.name:
            return 0
        return 1

class HiddenDepartment(NoDepartment):
    """ departement use when there is no need of departments """
    display = False


class Employee :
    "User shell class"
    
    def __init__(self, browseRec):
        'Constructor that take a User browse record'
        self.br = browseRec
        self.name = self.br.name 
        self.id = self.br.id
                
    def __cmp__(self, other):
        'comparator function'
        return cmp(self.name, other.name) 



class ChartByDepUsers(StandardReport): 
    """ display a chart of holidays: on X axis: time. On Y axis: departments and users """
    
    
    
    def get_holidays_from_to(self, holidays_objects):
        """ return the bounding dates of the holidays objects """
        
        #
        # find date_from and date_to from the holidays requests.
        #   
        date_from = None
        date_to = None     
        for i in holidays_objects:
            if date_from == None or date_from > i.date_from:
                date_from = i.date_from
            if date_to == None or date_to < i.date_to:
                date_to = i.date_to 
                
        if date_from == None:
            date_from = datetime.datetime.now()
        else: 
            date_from = datetime.datetime(*(time.strptime(date_from, "%Y-%m-%d %H:%M:%S")[0:3])) 
        if date_to == None:
            date_to = datetime.datetime.now()
        else:
            date_to   = datetime.datetime(*(time.strptime(date_to  , "%Y-%m-%d %H:%M:%S")[0:3]))
            
        return (date_from, date_to)
    
    
        
    def get_holidays_employees_ids(self, holidays_objects):
        """return a list of employees ids that are concerned by those holidas objects """
        
        employees_ids = []
        
        employee_obj = self.pool.get('hr.employee')
        for holidays in holidays_objects:
            # the employee is either defined in the holidays object
            if holidays.employee_id and (holidays.employee_id.id not in employees_ids):
                employees_ids.append(holidays.employee_id.id)
                
            # or only the user is defined and then, we have to find the coresponding employee(s)
            else: 
                employee_search = employee_obj.search(self.cr, self.uid, [('user_id', '=', holidays.user_id.id )])
                if employee_search:
                    search_result = employee_obj.browse(self.cr, self.uid, employee_search)
                    for tmp_employee in search_result:
                        if tmp_employee.id not in employees_ids:
                            employees_ids.append(tmp_employee.id)
                else:
                    #damn, this holiday request is not linked to anyone!
                    pass
        return employees_ids
        
        
        
        
    def get_employees_departments(self, employees_objects):
        """return departments that at least on employee belongs to """
        
        #get users related to employees
        users = []
        for e_id in employees_objects:
            e = employees_objects[e_id]
            if e.user_id:
                users.append(e.user_id)


        #
        #get departements related to users
        #
        departments_ids = [] 
        departments = {}
        department_obj = self.pool.get('hr.department')
        
        
        #find departments related to employees
        if len(users) > 0:
            query = """SELECT distinct(department_id)
                FROM hr_department_user_rel
                WHERE user_id IN (%s) ORDER BY department_id""" % ','.join(map(lambda x: str(x.id), users))
            self.cr.execute(query)
            res = self.cr.fetchall()     
            departments_ids = map(lambda x: x[0], res)
            
        departments_list = department_obj.browse(self.cr, self.uid, departments_ids)
        for d in departments_list:
            departments[d.id] = d
        
        return (departments_ids, departments)
        
        
    
    def get_story(self):
        """ return the elements of the report """
        story = []

        cr = self.cr
        context = self.context
        #
        # get report's bounding dates
        #
        
        #find holidays bounding dates
        date_from, date_to = self.get_holidays_from_to(self.objects)
        # date defined in the wizard override computed dates
        if 'form' in self.datas and 'date_from' in self.datas['form'] and self.datas['form']['date_from'] != False:
            date_from = self.datas['form']['date_from']
            date_from = datetime.datetime(*(time.strptime(date_from, "%Y-%m-%d")[0:3])) 
                   
        if 'form' in self.datas and 'date_to' in self.datas['form'] and self.datas['form']['date_to'] != False:
            date_to = self.datas['form']['date_to']
            date_to = datetime.datetime(*(time.strptime(date_to, "%Y-%m-%d")[0:3]))


        #we can already init the chart
        chart = TimeBoxesChartLegendBuilder(self._("Holidays")+" "+self._("From")+" "+date_from.strftime("%d.%m.%Y")+" "+self._("To")+" "+date_to.strftime("%d.%m.%Y"), self._("Types"), date_from, date_to)



        #
        # get all employees
        # three modes: - we get no list of employees => we have to find those who have holidays from the holidays objects
        #              - we get a list of employees and we want to display them all
        #              - we get a list of employees and we want to display only those who have holidays
        #
        
        employees_ids = []
        employees = {}
        
        #employees concerned by holidays    
        holiday_employees = self.get_holidays_employees_ids(self.objects)
        
        #employees are defined?
        form_employees = []
        if ('form' in self.datas) and ('employees' in self.datas['form']):    
            form_employees = self.datas['form']['employees']
           
            #mode one: employees are defined and we want only those with holidays
            if ('form' in self.datas) and ('only_holidays' in self.datas['form']) and self.datas['form']['only_holidays']:        
                employees_ids = list(set(form_employees) & set(holiday_employees))
            #mode two: employees are defined and we want to display them all
            else:
                employees_ids = form_employees            
              
        #mode three: employee are not defined
        else:        
            employees_ids = holiday_employees
        
            

        employee_obj = self.pool.get('hr.employee')
        employees_list = employee_obj.browse(self.cr, self.uid, employees_ids)
        for e in employees_list:
            employees[e.id] = e 
                        
                       
        departments_list = []
        employees_departments = {}

        
        #group by departments?
        if ('form' in self.datas) and ('group_by' in self.datas['form']) and (self.datas['form']['group_by'] == 'department'):



            #
            #get all departments
            #           
            departments_ids, departments = self.get_employees_departments(employees)
            
        
            #if we want to filter by department, keep only those selected in the wizard
            if 'form' in self.datas and 'depts' in self.datas['form'] and len(self.datas['form']['depts'][0][2]) > 0:            
                departments_ids = list(set(departments_ids) & set(self.datas['form']['depts'][0][2]))
                department_obj = self.pool.get('hr.department')
                departments_browse = department_obj.browse(self.cr, self.uid, departments_ids)
                for d in departments_browse:
                    departments[d.id] = d
    
                               
                               
            #list of employees not in a department
            orphans_employees = list(employees_ids)
            
            #let's build a list of departments and list of users
            for d_id in departments_ids:
                department_tmp = Department(departments[d_id])
                for e_id in employees_ids:
                    e = employees[e_id]
                    #does the employee belongs to this department?
                    if e.user_id and e.user_id in departments[d_id].member_ids:
                        #if so, he is not orphans
                        if e_id in orphans_employees:
                            orphans_employees.remove(e_id)
    
                        #init employees_departments list
                        if e_id not in employees_departments:
                            employees_departments[e_id] = []
                        #add the department to the employee
                        employees_departments[e_id].append(department_tmp)
                        
                        #add the employee to the department    
                        department_tmp.employees.append(Employee(e))
    
                departments_list.append(department_tmp)        
                
            #if there is no department selected as filters, show also orphans
            if 'form' not in self.datas or 'depts' not in self.datas['form'] or len(self.datas['form']['depts'][0][2]) == 0:                        
                
                #no child left behind! let's affect orphans employees to a dummy departments
                if orphans_employees:
                    noDepartment = NoDepartment()
                    for e_id in orphans_employees:
                        noDepartment.employees.append(Employee(employees[e_id]))
                        if e_id not in employees_departments:
                            employees_departments[e_id] = []
                        employees_departments[e_id].append(noDepartment)
                    departments_list.append(noDepartment)
                
            

            
        # do not diplay departments
        else: 
            #put all users in a invisible dummy department
            dummy_department = HiddenDepartment()
            for e_id in employees_ids:
                e = employees[e_id]
                dummy_department.employees.append(Employee(e))
                employees_departments[e_id] = [dummy_department]

            departments_list.append(dummy_department)
            




        #let's sort this mess!
        departments_list.sort()
        for d in departments_list: 
            d.employees.sort()

                               
        #hurra!! let's build the chart!
        #add elements on the Y axis 
        for d in departments_list:
            indent = 1
            if d.display:
                chart.append_Y_item('<b>'+d.name+'</b>', indent, "dep_"+str(d.id))
                indent = 2
            for e in d.employees:
                chart.append_Y_item(e.name, indent, "dep_"+str(d.id)+"user_"+str(e.id))

                
        #add periods
        for holidays in self.objects:
            holidays_date_from = datetime.datetime(*(time.strptime(holidays.date_from, "%Y-%m-%d %H:%M:%S")[0:3])) 
            holidays_date_to   = datetime.datetime(*(time.strptime(holidays.date_to  , "%Y-%m-%d %H:%M:%S")[0:3]))

            # the employee is either defined in the holiday...
            employee_id = None
            if holidays.employee_id:
                employee_id = holidays.employee_id.id
            # ...or only the user is defined and then, we have to find the coresponding employee
            else: 
                employee_ids = employee_obj.search(self.cr, self.uid, [('user_id', '=', holidays.user_id.id )])
                if employee_ids and len(employee_ids) > 0:
                    employee = employee_obj.browse(self.cr, self.uid, employee_ids[0])
                    employee_id = employee.id
                else:
                    #damn, this holiday request is not linked to anyone!
                    pass
                
            if employee_id != None and employee_id in employees_departments:
                for d in employees_departments[employee_id]:
                    grid = None
                    name = holidays.name
                    #holiday not validated
                    if holidays.state in ['cancel', 'confirm']: 
                        grid = "black"
                        name = "? "+name
                    
                    #no color defined
                    if holidays.holiday_status.color_name == False:
                        color = None
                    #color defined
                    else: 
                        color = holidays.holiday_status.color_name
                    chart.add_period("dep_"+str(d.id)+"user_"+str(employee_id), holidays_date_from, holidays_date_to, color, name, holidays.holiday_status.name, grid)         

        
        story.append(chart.get_flowable(self.width))
        story.append(chart.get_legend_flowable(self.width))

        return story


ChartByDepUsers('report.holidays_by_dep_users', "Holidays", "hr.holidays", StandardReport.A4_LANDSCAPE)            
