# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) Camptocamp SA
# Author: Arnaud Wüst
#
#
#    This file is part of the c2c_holidays_report module
#
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import wizard
import datetime
import time
import pooler

form='''<?xml version="1.0"?>
<form string="Report Options" height="200" width="200">
    <group colspan="4" col="4" string="Limits">
        <field name="date_from" colspan="2" />
        <field name="date_to" colspan="2" />
    </group>
    <group colspan="4" col="2" string="Filters">
        <field name="only_holidays" colspan="2" />
        <field name="holiday_type" colspan="2" />
    </group>
</form>'''


class wizard_report(wizard.interface):
    
    
    def _init(self, cr, uid, data, context):
        
        result = {}
        
        #init
        result['date_from']=time.strftime('%Y-%m-%d')
        result['holiday_type']='Validated'
        result['only_holidays'] = True
        
        #get holidays
        holidays_obj = pooler.get_pool(cr.dbname).get('hr.holidays')
        ids = holidays_obj.search(cr, uid, [])
        obj = holidays_obj.browse(cr, uid, ids)
        
        #find the end date
        end_date= time.strftime('%Y-%m-%d')
        for o in obj: 
            if end_date < o.date_to:
                end_date = o.date_to.split()[0]
        result['date_to']=end_date
        return result


    def _get_ids(self, cr, uid, data, context):
        
        result = {}
        
        #date start before date end? 
        if data['form']['date_from'] > data['form']['date_to']:
           raise wizard.except_wizard('Date error:', 'Date from must preceed date to. Try again with different dates.')  
       
        #more than 2000 days between dates?
        d_from = datetime.datetime(*(time.strptime(data['form']['date_from'], "%Y-%m-%d")[0:3])) 
        d_to = datetime.datetime(*(time.strptime(data['form']['date_to'], "%Y-%m-%d")[0:3]))  
        days_distance = d_to - d_from
        if  days_distance.days > 2000:
           raise wizard.except_wizard('Date error:', 'Too much distance between dates: %s days (max 2000 days). Try again with closer dates. ' % days_distance.days)  
        
        
        if data['form']['holiday_type'] == 'Confirmed':
            states = ['confirm']
        elif data['form']['holiday_type'] == 'Validated':
            states = ['validate']
        else:
            states = ['confirm', 'validate']
    
        
        holidays_obj = pooler.get_pool(cr.dbname).get('hr.holidays')
        ids = holidays_obj.search(cr, uid, [('date_to', '>', data['form']['date_from']), ('date_from', '<', data['form']['date_to']), ('state', 'in', states)])
#        if len(ids) == 0:
#           raise wizard.except_wizard('Filters error:', 'Your settings return no results. Modify your criteria to enlarge your selection')  
        result['ids'] = ids     
        
        result['employees'] = data['ids'] 
            
        return result
        

    fields={
        'date_from':{
                'string':'From',
                'type':'date',
                'required': True,
        },
        'date_to':{
                'string':'To',
                'type':'date',
                'required': True,
        },
        'only_holidays' : {
                'string':'Display only employees\n who have planned holidays', 
                'type':'boolean'
        },        
        'depts': {
                'string': '', 
                'type': 'many2many', 
                'relation': 'hr.department'
        },
        'holiday_type':{
                'string':"Select Holiday Status",
                'required':True,
                'type':'selection',
                'selection':[('Validated','Validated (?)'),('Confirmed','Confirmed'),('both','Both Validated (?) and Confirmed')]
        },
    }

    states={
        'init':{
            'actions':[_init],
            'result':{'type':'form', 'arch':form, 'fields':fields, 'state':[('end', 'Cancel'), ('report', 'Print')]}
        },
        'report':{
            'actions':[_get_ids],
            'result':{'type':'print', 'report':'holidays_by_dep_users', 'get_id_from_action':True, 'state':'end'}
        }
    }
wizard_report('c2c_holidays.chart_from_employees')
