#!/usr/bin/env python2.3
#
#  account.py
#  conventus
#
#  Created by Nicolas Bessi on 19.06.08.
#  Copyright (c) 2008 CamptoCamp. All rights reserved.
#


from osv import osv, fields
import pooler

class account_move_line(osv.osv):
	""" Inherit of account_move_line in order to auto compute the currency amount"""
	_inherit = "account.move.line"
	_description = "Entry lines"
	
	def fields_view_get(self, cr, uid, view_id=None, view_type='form', context={}, toolbar=False):
		result = super(osv.osv, self).fields_view_get(cr, uid, view_id, view_type, context)
		if view_type=='tree' and 'journal_id' in context:
			title = self.view_header_get(cr, uid, view_id, view_type, context)
			journal = self.pool.get('account.journal').browse(cr, uid, context['journal_id'])

			# if the journal view has a state field, color lines depending on
			# its value
			state = ''
			for field in journal.view_id.columns_id:
				if field.field=='state':
					state = ' colors="red:state==\'draft\'"'
			xml = '''<?xml version="1.0"?>\n<tree string="%s" editable="top" refresh="5" on_write="_on_create_write"%s>\n\t''' % (title, state)
			fields = []

			widths = {
				'ref': 50,
				'statement_id': 50,
				'state': 60,
				'tax_code_id': 50,
				'move_id': 40,
			}
			# we dynamically generate the view and adde the on change on colums 
			for field in journal.view_id.columns_id:
				fields.append(field.field)
				attrs = []
				if field.field == 'debit':
					attrs.append('sum="Total debit"')
					attrs.append('on_change="on_change_deb( debit, credit, currency_id, amount_currency, date_maturity, partner_id)"')
				elif field.field == 'credit':
					attrs.append('sum="Total credit"')
					attrs.append('on_change="on_change_cred( debit, credit, currency_id, amount_currency, date_maturity, partner_id)"')
				elif field.field == 'amount_currency':
					attrs.append('sum="Total amount_currency"')
					attrs.append('on_change="on_change_amountcur( debit, credit, currency_id, amount_currency, date_maturity, partner_id)"')
				elif field.field == 'currency_id':
					attrs.append(' on_change="on_change_my_c2c_currency(debit, credit, currency_id, amount_currency, date_maturity, partner_id)" ')
				elif field.field == 'account_id' and journal.id:
					attrs.append('domain="[(\'journal_id\', \'=\', '+str(journal.id)+'),(\'type\',\'&lt;&gt;\',\'view\'), (\'type\',\'&lt;&gt;\',\'closed\')]"')
				if field.readonly:
					attrs.append('readonly="1"')
				if field.required:
					attrs.append('required="1"')
				else:
					attrs.append('required="0"')
				if field.field == 'partner_id':
					attrs.append('on_change="onchange_partner_id(move_id,partner_id,account_id,debit,credit,((\'journal_id\' in context) and context[\'journal_id\']) or {})"')
				if field.field in widths:
					attrs.append('width="'+str(widths[field.field])+'"')
				xml += '''<field name="%s" %s/>\n''' % (field.field,' '.join(attrs))

			xml += '''</tree>'''
			result['arch'] = xml
			result['fields'] = self.fields_get(cr, uid, fields, context)
		return result

	
	
	def on_change_my_c2c_currency(self, cr, uid, ids, debit, credit, currency_id, amount_currency, date_maturity, partner_id) :
		"""Function call if the currency is changed"""
		res = {}
		context = {}
		if currency_id :
			if date_maturity :
				context['date'] = date_maturity
			res_currency_obj = self.pool.get('res.currency').browse(cr, uid, currency_id, context)
			comp_currency = self.pool.get('res.users').browse(cr, uid, uid, context).company_id.currency_id.id
			if amount_currency :
				if credit :
					credit = res_currency_obj.compute(cr, uid, comp_currency, currency_id, amount_currency, True,context)*-1
				if debit :
					debit = res_currency_obj.compute(cr, uid, comp_currency, currency_id, amount_currency, True, context)
			else :
				if credit :
					amount_currency = res_currency_obj.compute(cr, uid, comp_currency, currency_id, credit, True, context)
				if debit :
					amount_currency = res_currency_obj.compute(cr, uid, comp_currency, currency_id, debit, True, context)
		return {'value': {
				'debit': debit,
				'credit' :credit,
				'amount_currency': amount_currency,
				
				}}
	
	def on_change_deb(self, cr, uid, ids, debit, credit, currency_id, 
		amount_currency, date_maturity, partner_id):
		"""function call if the debit value is changed set the credit to zero"""
		to_return = self.on_change_debcred(cr, uid, ids, debit, 
			credit, currency_id, amount_currency, date_maturity, partner_id) 
		to_return['value']['credit'] = 0
		return to_return 

	
	def on_change_cred(self, cr, uid, ids, debit, credit, currency_id, 
		amount_currency, date_maturity, partner_id):
		"""function call if the credit value is changed set the debit to zero"""

		to_return = self.on_change_debcred(cr, uid, ids, debit, 
			credit, currency_id, amount_currency, date_maturity, partner_id) 
		to_return['value']['debit'] = 0
		return to_return 

	def on_change_debcred(self, cr, uid, ids, debit, credit, currency_id, 
		amount_currency, date_maturity, partner_id) :
		res = {}
		context = {}
		if currency_id :
			# if there is a maturity date it will be used for getting the currency
			if date_maturity :
				context['date'] = date_maturity
			res_currency_obj = self.pool.get('res.currency').browse(cr, uid, currency_id, context)
			comp_currency = self.pool.get('res.users').browse(cr, uid, uid, context).company_id.currency_id.id
			#if the sum is in cedit the output will be negative
			if credit :
				amount_currency = res_currency_obj.compute(cr, uid, comp_currency, currency_id, credit, True, context)*-1
			#if the sum is in debit the output will be positive
			if debit :
				amount_currency = res_currency_obj.compute(cr, uid, comp_currency, currency_id, debit, True, context)
		
		return {'value': {
				'debit': debit,
				'credit' :credit,
				'amount_currency': amount_currency,
				
				}}
				
				
	def on_change_amountcur(self, cr, uid, ids, debit, credit, currency_id, 
		amount_currency, date_maturity, partner_id) :
		"""function call if the amount currency is changed"""
		res = {}
		context = {}
		if currency_id and not credit and not debit:
		# if there is a maturity date it will be used for getting the currency

			if date_maturity :
				context['date'] = date_maturity
			res_currency_obj = self.pool.get('res.currency').browse(cr, uid, currency_id, context)
			comp_currency = self.pool.get('res.users').browse(cr, uid, uid, context).company_id.currency_id.id
			# if amount_currency is positiv it will go on debit else on credit
			if amount_currency <= 0 :
				credit = abs(res_currency_obj.compute(cr, uid, currency_id, comp_currency, amount_currency, True, context))
			else :
				debit = abs(res_currency_obj.compute(cr, uid,  currency_id, comp_currency, amount_currency, True, context))
		
		return {'value': {
				'debit': debit,
				'credit' :credit,
				'amount_currency': amount_currency,
				
				}}


account_move_line()