# -*- coding: utf-8 -*- 
##############################################################################
#
# Copyright (c) Camptocamp SA - Joel Grand-Guillaume
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
import time
from mx import DateTime
import netsvc
from osv import fields
from osv import osv
from datetime import datetime



# Build a view for each day, with an amount of avalaible hours per product
class report_analytic_c2c_planning_available_byprod(osv.osv):
	_name = "report_account_analytic.c2c.planning.available.byprod"
	_description = "Planning avalaible by product"
	_rec_name = 'date'
	_auto = False
	_log_access = False
	_order = 'date,product_id'
	
	def _get_week_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_week, "%Y-%m-%d")[0:6])).strftime("%Y-W%W")
		return result

	def _get_month_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_month, "%Y-%m-%d")[0:6])).strftime("%Y-%m")
			#DateTime.strptime(line.date_month,"%Y-%m-%d").strftime("%B %Y")”
			
		return result

	_columns = {
		'product_id': fields.many2one('product.product', 'Function (Product)'),
		'regime': fields.float('Avaliable'),
		'date': fields.date('Day'),
		'date_week': fields.date('Week dates'),
		'date_month': fields.date('Month dates'),
		'week': fields.function(_get_week_name, type="char",method=True, string='Week'),
		'month': fields.function(_get_month_name, type="char",method=True, string='Month'),
	}
	_order = 'product_id'
	def init(self, cr):
		# Generate series for working days between dates
		cr.execute("""CREATE OR REPLACE FUNCTION non_weekends_between(
			first_date DATE, /* $1 */
			last_date DATE   /* $2 */
		)
		RETURNS SETOF date
		LANGUAGE sql
		AS
		$$
		SELECT
			$1 + s.i
		FROM generate_series(
			0,
			$2 - $1
		) AS s(i)
		WHERE
			extract(
				DOW
			FROM
				$1 + s.i
			) NOT IN (
				0,  /* Sunday   */
				6   /* Saturday */
			);
		$$;
		""")
		# count working day between dates
		cr.execute("""CREATE OR REPLACE FUNCTION working_day_in_dates (
			first_date DATE, /* $1 */
			last_date DATE   /* $2 */   )
		RETURNS int8
		LANGUAGE sql
		AS
		$$
		select count(*) 
		from(
			select non_weekends_between($1,$2)  as foo
		) as foo2
		$$;
		""")
		# generate available ressources for futur 6 months (with active=true and date in hr_employee are in scope)
		cr.execute("""create or replace view report_analytic_c2c_planning_available_byprod as (
			select 
				/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||e product_id)
			  	Ok from year 2000 to 2020 and 999'999 entries of employee*/
				(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(e.product_id,0)::varchar)::int as id,
				sum(e.regime)/5 as regime,
				e.product_id as product_id,
				series.date as date,
				date_trunc ('week', series.date)::date as date_week,
				date_trunc ('month', series.date)::date as date_month
			from (
			        select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
			            /*select date_from,date_to from report_account_analytic_planning_line*/
			            ) as date
			    ) as series
			    /*select lines that cross the date, like in advanced planning line search wizard*/
			left outer join hr_employee e on (series.date >= e.started or e.started is null) and (series.date <= e.ended or e.ended is null)
			where e.active='t'
			group by series.date, e.product_id
			order by series.date
		);""")
report_analytic_c2c_planning_available_byprod()

# Build a view for each day, with an amount of avalaible hours per user
class report_analytic_c2c_planning_available_byuser(osv.osv):
	_name = "report_account_analytic.c2c.planning.available.byuser"
	_description = "Planning avalaible by user"
	_rec_name = 'date'
	_auto = False
	_log_access = False
	_order = 'date,user_id'
	
	def _get_week_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_week, "%Y-%m-%d")[0:6])).strftime("%Y-W%W")
		return result

	def _get_month_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_month, "%Y-%m-%d")[0:6])).strftime("%Y-%m")
			#DateTime.strptime(line.date_month,"%Y-%m-%d").strftime("%B %Y")”
			
		return result

	_columns = {
		'user_id': fields.many2one('res.users', 'User'),
		'regime': fields.float('Avaliable'),
		'date': fields.date('Day'),
		'date_week': fields.date('Week dates'),
		'date_month': fields.date('Month dates'),
		'week': fields.function(_get_week_name, type="char",method=True, string='Week'),
		'month': fields.function(_get_month_name, type="char",method=True, string='Month'),
	}
	_order = 'user_id'
	def init(self, cr):
				
		# generate available ressources for futur 6 months (with active=true and date in hr_employee are in scope)
		cr.execute("""create or replace view report_analytic_c2c_planning_available_byuser as (
			select 
				/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||e user_id)
			  	Ok from year 2000 to 2020 and 999'999 entries of employee*/
				(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(e.user_id,0)::varchar)::int as id,
				sum(e.regime)/5 as regime,
				e.user_id as user_id,
				series.date as date,
				date_trunc ('week', series.date)::date as date_week,
				date_trunc ('month', series.date)::date as date_month
			from (
			        select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
			            /*select date_from,date_to from report_account_analytic_planning_line*/
			            ) as date
			    ) as series
			    /*select lines that cross the date, like in advanced planning line search wizard*/
			left outer join hr_employee e on (series.date >= e.started or e.started is null) and (series.date <= e.ended or e.ended is null)
			where e.active='t'
			group by series.date, e.user_id
		);""")
report_analytic_c2c_planning_available_byuser()
#######################################################################################
#  the incredible view that generate one planning line a day, with amount for each one,
#  take care of non working days (saturday,sunday) by JGE
#######################################################################################
# HELP => Lokok at the bottom, there is part of reflexion...

class report_account_analytic_c2c_planning_line_series(osv.osv):
	_name = "report_account_analytic.c2c.planning.line.series"
	_description = "Planning lines series stats"
	_rec_name = 'account_id'
	_auto = False
	_log_access = False
	_order = 'date,date_week,date_month'
	
	def _get_week_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_week, "%Y-%m-%d")[0:6])).strftime("%Y-W%W")
		return result

	def _get_month_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_month, "%Y-%m-%d")[0:6])).strftime("%Y-%m")
			#DateTime.strptime(line.date_month,"%Y-%m-%d").strftime("%B %Y")”
			
		return result

	# Take the sum of hours available for each product
	# for each line (regime in hr_employee / 5 / count)
	# def _get_product_amount_prod_a_day(self, cr, uid, ids, name, args, context):
	# 	result = {}
	# 	for id in ids:
	# 		result[id] = 0.0
	# 	ids_set = ",".join(map(str, ids))
	# 	cr.execute("select sum(regime)/5 as amount,product_id from hr_employee group by product_id")
	# 	res_regime=cr.dictfetchall()
	# 	dict_regime_product={}
	# 	for regime in res_regime:
	# 		dict_regime_product[regime['product_id']]=regime['amount']
	# 	cr.execute("select id,product_id,count_prod as count_prod from report_account_analytic_c2c_planning_line_series")
	# 	res_lines=cr.dictfetchall()
	# 	for line in res_lines:
	# 		if dict_regime_product.has_key(line['product_id']):
	# 			result[line['id']]=dict_regime_product[line['product_id']]/line['count_prod']
	# 		else:
	# 			result[line['id']]=0.0
	# 	
	# 	return result
	# # Take the sum of hours available for each user
	# # for each line (regime in hr_employee / 5 / count)
	# def _get_product_amount_user_a_day(self, cr, uid, ids, name, args, context):
	# 	result = {}
	# 	for id in ids:
	# 		result[id] = 0.0
	# 	ids_set = ",".join(map(str, ids))
	# 	cr.execute("select sum(regime)/5 as amount,user_id from hr_employee group by user_id")
	# 	res_regime=cr.dictfetchall()
	# 	dict_regime_product={}
	# 	for regime in res_regime:
	# 		dict_regime_product[regime['user_id']]=regime['amount']
	# 	cr.execute("select date,user_id,sum(amount) as amount_planned from report_account_analytic_c2c_planning_line_series group by user_id,date")
	# 	res_lines=cr.dictfetchall()
	# 	for line in res_lines:
	# 		if dict_regime_product.has_key(line['user_id']):
	# 			result[line['id']]=dict_regime_product[line['user_id']]-line['amount_planned']
	# 		else:
	# 			result[line['id']]=0.0
	# 	
	# 	return result
		
		
	
		
	_columns = {
		'planning_id': fields.many2one('report_account_analytic.planning', 'Planning'),
		'user_id': fields.many2one('res.users', 'User'),
		'manager_id': fields.many2one('res.users', 'Manager'),
		'department_id': fields.many2one('hr.department', 'Departement'),
		'account_id': fields.many2one('account.analytic.account', 'Account'),
		'product_id': fields.many2one('product.product', 'Function (Product)'),
		'activity_id': fields.many2one('project.activity_al', 'Activity'),
		'amount': fields.float('Planned hours'),
		# 'count_prod': fields.float('Nb of this product for the day'),
		# 'count_user': fields.float('Nb user in line for the day'),
		'date': fields.date('Day'),
		'date_week': fields.date('Week dates'),
		'date_month': fields.date('Month dates'),
		'week': fields.function(_get_week_name, type="char",method=True, string='Week'),
		'month': fields.function(_get_month_name, type="char",method=True, string='Month'),
		'regime_prod': fields.float('Regime for date/product'),
		'regime_user': fields.float('Regime for date/user'),
	}
	_order = 'planning_id,user_id'
	
	# For all stat, take only default planning !
	
	def init(self, cr):
				
		# generate empty view (if nothing exits because of cylcic join)
		cr.execute("""create or replace view report_account_analytic_c2c_planning_line_series_simplified as (
			select 
				/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||line id)
				  Ok from year 2000 to 2020 and 999'999 entries of planning lines*/
				(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(l.id,0)::varchar)::int as id,
				date_trunc ('week', series.date)::date as date_week,
				date_trunc ('month', series.date)::date as date_month,
				series.date as date,
				/*divid unified_amount by working day*/
				(l.unified_amount*uo.factor)/working_day_in_dates(min(l.date_from),max(l.date_to))::float as amount,
				l.planning_id as planning_id,
				l.manager as manager_id,
				l.account_id as account_id,
				l.department_id as department_id,
				COALESCE(l.user_id, 0) as user_id,
				l.product_id as product_id,
				l.activity_id as activity_id,
			from (
			        /*take working day from begin of month to 6 month*/
					select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
						) as date
				) as series
				/*select lines that cross the date, like in advanced planning line search wizard*/
			left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to)
			join report_account_analytic_planning pl on (pl.id=l.planning_id) join product_uom uo on (uo.id=l.amount_unit)
			where pl.state='open' and default_planning = 't'
			group by series.date,l.id,l.unified_amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id, l.amount_unit,uo.factor
			order by series.date,date_week,date_month,product_id
		);""")
		# generate sum planed for user a day
		cr.execute("""create or replace view report_account_analytic_c2c_planning_line_series_init2 as (
				/* Take lines with a user */
				select 
					/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||line id)
					  Ok from year 2000 to 2020 and 999'999 entries of planning lines*/
					(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(l.id,0)::varchar)::int as id,
					date_trunc ('week', series.date)::date as date_week,
					date_trunc ('month', series.date)::date as date_month,
					series.date as date,
					/*divid unified_amount by working day*/
					(l.unified_amount*uo.factor)/working_day_in_dates(min(l.date_from),max(l.date_to))::float as amount,
					l.planning_id as planning_id,
					l.manager as manager_id,
					l.account_id as account_id,
					l.department_id as department_id,
					COALESCE(l.user_id, 0) as user_id,
					l.product_id as product_id,
					/*Number of this user_id for this day*/
					sum (thisview.amount)::float as available_amount_user,
					l.activity_id as activity_id
				from (
				        /*take working day from now to 6 month*/
						select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
							) as date
					) as series
					/*select lines that cross the date, like in advanced planning line search wizard*/
				left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to)
				join report_account_analytic_planning pl on (pl.id=l.planning_id) join product_uom uo on (uo.id=l.amount_unit)
				,report_account_analytic_c2c_planning_line_series_simplified thisview
				where thisview.date=series.date and thisview.user_id = l.user_id
				and pl.state='open' and default_planning = 't'
				group by series.date,l.id,l.unified_amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id,l.amount_unit,uo.factor
				order by series.date,date_week,date_month,user_id
		);""")	
		# generate count for product_id a day
		cr.execute("""create or replace view report_account_analytic_c2c_planning_line_series_init3 as (
			select 
				/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||line id)
				  Ok from year 2000 to 2020 and 999'999 entries of planning lines*/
				(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(l.id,0)::varchar)::int as id,
				date_trunc ('week', series.date)::date as date_week,
				date_trunc ('month', series.date)::date as date_month,
				series.date as date,
				/*divid unified_amount by working day*/
				(l.unified_amount*uo.factor)/working_day_in_dates(min(l.date_from),max(l.date_to))::float as amount,
				l.planning_id as planning_id,
				l.manager as manager_id,
				l.account_id as account_id,
				l.department_id as department_id,
				COALESCE(l.user_id, 0) as user_id,
				l.product_id as product_id,
				/*Number of this user_id for this day*/
				sum (thisview.amount)::float as available_amount_prod,
				l.activity_id as activity_id
			from (
			        /*take working day from now to 6 month*/
					select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
						) as date
				) as series
				/*select lines that cross the date, like in advanced planning line search wizard*/
			left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to)
			join report_account_analytic_planning pl on (pl.id=l.planning_id) join product_uom uo on (uo.id=l.amount_unit)
			,report_account_analytic_c2c_planning_line_series_simplified thisview
			where thisview.date=series.date and thisview.product_id = l.product_id
			and pl.state='open' and default_planning = 't'
			group by series.date,l.id,l.unified_amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id,l.amount_unit,uo.factor
			order by series.date,date_week,date_month,product_id
		);""")
		# Planning for 6 month (with count for user and product)
		cr.execute("""
			create or replace view report_account_analytic_c2c_planning_line_series as (
				select 
					/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||line id)
					  Ok from year 2000 to 2020 and 999'999 entries of planning lines*/
					(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(l.id,0)::varchar)::int as id,
					date_trunc ('week', series.date)::date as date_week,
					date_trunc ('month', series.date)::date as date_month,
					series.date as date,
					/*divid unified_amount by working day*/
					(l.unified_amount*uo.factor)/working_day_in_dates(min(l.date_from),max(l.date_to))::float as amount,
					l.planning_id as planning_id,
					l.manager as manager_id,
					l.account_id as account_id,
					l.department_id as department_id,
					COALESCE(l.user_id, 0)  as user_id,
					l.product_id as product_id,
					l.activity_id as activity_id,
					regime.regime as regime_prod,
					regime_usr.regime as regime_user
				from (
				        /*take working day from now to 6 month*/
						select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
							) as date
					) as series
					/*select lines that cross the date, like in advanced planning line search wizard*/
				left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to)
				join report_analytic_c2c_planning_available_byprod regime on (regime.date=series.date and regime.product_id=l.product_id)
				join report_analytic_c2c_planning_available_byuser regime_usr on (regime_usr.date=series.date and regime_usr.user_id=l.user_id)
				join report_account_analytic_planning pl on (pl.id=l.planning_id) join product_uom uo on (uo.id=l.amount_unit)
				where pl.state='open' and default_planning = 't'
				group by series.date,l.id,l.unified_amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id,regime.regime,regime_usr.regime,l.amount_unit,uo.factor
				order by series.date,date_week,date_month,product_id,user_id
			);
		""")
		# Planning for 6 month (with count for user and product) fopr excel TODO
		# cr.execute("""
		# 		create or replace view report_account_analytic_c2c_planning_line_series_excel as (
		# 			select 
		# 				/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||line id)
		# 				  Ok from year 2000 to 2020 and 999'999 entries of planning lines*/
		# 				(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(l.id,0)::varchar)::int as id,
		# 				date_trunc ('week', series.date)::date as date_week,
		# 				date_trunc ('month', series.date)::date as date_month,
		# 				series.date as date,
		# 				/*divid unified_amount by working day*/
		# 				l.unified_amount/working_day_in_dates(min(l.date_from),max(l.date_to))::float as amount,
		# 				l.planning_id as planning_id,
		# 				l.manager as manager_id,
		# 				l.account_id as account_id,
		# 				l.department_id as department_id,
		# 				COALESCE(u.name, 'None')  as user_id,
		# 				l.product_id as product_id,
		# 				l.activity_id as activity_id,
		# 				regime.regime as regime_prod,
		# 				regime_usr.regime as regime_user
		# 			from (
		# 			        /*take working day from now to 6 month*/
		# 					select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
		# 						) as date
		# 				) as series
		# 				/*select lines that cross the date, like in advanced planning line search wizard*/
		# 			left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to)
		# 			join report_analytic_c2c_planning_available_byprod regime on (regime.date=series.date and regime.product_id=l.product_id)
		# 			join report_analytic_c2c_planning_available_byuser regime_usr on (regime_usr.date=series.date and regime_usr.user_id=l.user_id)
		# 			join res_users u on (u.id=l.user_id),
		# 			join product_product u on (u.id=l.user_id),
		# 			group by series.date,l.id,l.unified_amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id,regime.regime,regime_usr.regime
		# 			order by series.date,date_week,date_month,product_id,user_id
		# 		);
		# 	""")
		# Planning for 6 month (with count for user and product) with all computed infos
		cr.execute("""
			create or replace view report_account_analytic_c2c_planning_line_series_complete as (
				select 
					/*Generate series for unique id, based on line numer date from sries (take (number of days till 1970 - 10000)||line id)
					  Ok from year 2000 to 2020 and 999'999 entries of planning lines*/
					(((((EXTRACT ('epoch' FROM series.date)/3600)/24)-11000::int)::int)::varchar|| COALESCE(l.id,0)::varchar)::int as id,
					date_trunc ('week', series.date)::date as date_week,
					date_trunc ('month', series.date)::date as date_month,
					series.date as date,
					/*divid unified_amount by working day*/
					(l.unified_amount*uo.factor)/working_day_in_dates(min(l.date_from),max(l.date_to))::float as amount,
					l.planning_id as planning_id,
					l.manager as manager_id,
					l.account_id as account_id,
					l.department_id as department_id,
					COALESCE(l.user_id, 0)  as user_id,
					l.product_id as product_id,
					l.activity_id as activity_id,
					/*Number of this product for this day*/
					min(thisview2.available_amount_user) as available_amount_user,
					min(thisview3.available_amount_prod) as available_amount_prod,
					regime.regime as regime_prod,
					regime_usr.regime as regime_user
				from (
				        /*take working day from now to 6 month*/
						select non_weekends_between(date_trunc('month',current_date)::date,(current_date+ interval '4 month')::date
							) as date
					) as series
					/*select lines that cross the date, like in advanced planning line search wizard*/
				left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to)
				join report_analytic_c2c_planning_available_byprod regime on (regime.date=series.date and regime.product_id=l.product_id)
				join report_analytic_c2c_planning_available_byuser regime_usr on (regime_usr.date=series.date and regime_usr.user_id=l.user_id)
				join report_account_analytic_planning pl on (pl.id=l.planning_id) join product_uom uo on (uo.id=l.amount_unit)
				,report_account_analytic_c2c_planning_line_series_init2 thisview2
				,report_account_analytic_c2c_planning_line_series_init3 thisview3
				where thisview2.date=series.date and thisview2.user_id = l.user_id and 
				thisview3.date=series.date and thisview3.product_id = l.product_id 
				and pl.state='open' and default_planning = 't'
				group by series.date,l.id,l.unified_amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id,regime.regime,regime_usr.regime,l.amount_unit,uo.factor
				order by series.date,date_week,date_month,product_id,user_id
			);
		""")
report_account_analytic_c2c_planning_line_series()


class report_analytic_c2c_planning_compared_available_plan_byprod(osv.osv):
	_name = "report_analytic.c2c.planning.compared.available.plan.byprod"
	_description = "Planning avalaible"
	_rec_name = 'date'
	_auto = False
	_log_access = False
	_order = 'date,product_id'
	
	def _get_week_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_week, "%Y-%m-%d")[0:6])).strftime("%Y-W%W")
		return result

	def _get_month_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_month, "%Y-%m-%d")[0:6])).strftime("%Y-%m")
			#DateTime.strptime(line.date_month,"%Y-%m-%d").strftime("%B %Y")”
			
		return result

	_columns = {
		'product_id': fields.many2one('product.product', 'Function (Product)'),
		'avalaible_date': fields.float('Avaliable'),
		'amount_planned': fields.float('Planned'),
		'difference': fields.float('Difference'),
		'date': fields.date('Day'),
		'date_week': fields.date('Week dates'),
		'date_month': fields.date('Month dates'),
		'week': fields.function(_get_week_name, type="char",method=True, string='Week'),
		'month': fields.function(_get_month_name, type="char",method=True, string='Month'),
	}
	_order = 'product_id'
	def init(self, cr):

		cr.execute("""create or replace view report_analytic_c2c_planning_compared_available_plan_byprod as(
		select 
			min(a.id) as id,
			a.product_id as product_id,
			a.date as date,
			date_trunc ('week', a.date)::date as date_week,
			date_trunc ('month', a.date)::date as date_month,
			regime as avalaible_date,
			COALESCE(sum(amount),0.0)::float as amount_planned,
			regime-COALESCE(sum(amount),0.0) as difference
		from report_analytic_c2c_planning_available_byprod a left outer join report_account_analytic_c2c_planning_line_series_simplified s on (a.date=s.date and a.product_id=s.product_id)
		group by a.product_id, a.date, regime
		order by a.date
		);""")
report_analytic_c2c_planning_compared_available_plan_byprod()

class report_analytic_c2c_planning_compared_available_plan_byuser(osv.osv):
	_name = "report_analytic.c2c.planning.compared.available.plan.byuser"
	_description = "Planning avalaible"
	_rec_name = 'date'
	_auto = False
	_log_access = False
	_order = 'date,user_id'
	
	def _get_week_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_week, "%Y-%m-%d")[0:6])).strftime("%Y-W%W")
		return result

	def _get_month_name(self, cr, uid, ids, name, args, context):
		result = {}
		for line in self.browse(cr, uid, ids, context):
			result[line.id] = datetime(*(time.strptime(line.date_month, "%Y-%m-%d")[0:6])).strftime("%Y-%m")
			#DateTime.strptime(line.date_month,"%Y-%m-%d").strftime("%B %Y")”
			
		return result

	_columns = {
		'user_id': fields.many2one('res.users', 'User'),
		'avalaible_date': fields.float('Avaliable'),
		'amount_planned': fields.float('Planned'),
		'difference': fields.float('Difference'),
		'date': fields.date('Day'),
		'date_week': fields.date('Week dates'),
		'date_month': fields.date('Month dates'),
		'week': fields.function(_get_week_name, type="char",method=True, string='Week'),
		'month': fields.function(_get_month_name, type="char",method=True, string='Month'),
	}
	_order = 'user_id'
	def init(self, cr):
		# difference between planned and avalaible
		cr.execute("""create or replace view report_analytic_c2c_planning_compared_available_plan_byuser as(
		select 
			min(a.id) as id,
			a.user_id as user_id,
			a.date as date,
			date_trunc ('week', a.date)::date as date_week,
			date_trunc ('month', a.date)::date as date_month,
			regime as avalaible_date,
			COALESCE(sum(amount),0.0)::float as amount_planned,
			regime-COALESCE(sum(amount),0.0) as difference
		from report_analytic_c2c_planning_available_byuser a left outer join report_account_analytic_c2c_planning_line_series_simplified s on (a.date=s.date and a.user_id=s.user_id)
		group by a.user_id, a.date,regime
		order by a.date
		);""")
report_analytic_c2c_planning_compared_available_plan_byuser()

# 
# select *,regime/count as result from report_account_analytic_c2c_planning_line_series_available a join report_account_analytic_c2c_planning_line_series s on (a.date=s.date and a.product_id=s.product_id)


# PLANNING LINE VIEW:
# -------------------
# -------------------
# 
# => problem : define how to determine qtt per periode... baseed on : start_date, repartition between date, end date, both start and end date...
# 
# proposition:
# -------------------
# We must create one line per day in a view, and attribute a value for week and month based on this date. The unified amount of each line is computed like this:
# 
# Take the amount between start and end dates / number of (working ?) day between dates
# 
# When grouping on week / month / day, we will obtain the sum of unified amount
# 
# The view contains:
# -------------------
# planning_id, manager_id, account_id, department_id, user_id, plan_qtt_unified, prod_id, activiy, Month, week, date
# 
# 
# 
# 
# 
# Generating series:
# 
# tinyerp_c2c_jge=# select * from non_weekends_between('2008-10-20','2008-10-26');
#  non_weekends_between 
# ----------------------
#  2008-10-20
#  2008-10-21
#  2008-10-22
#  2008-10-23
#  2008-10-24
# (5 rows)
# 
# # Make a list of working days and generate planning infos for line
# 
# -- select series.date, count(sales.date)
# -- from (select generate_series(0,4) + date '2008-05-05' as date) as series
# -- left outer join sales on series.date=sales.date group by series.date
# -- order by series.date;
# 
# => problem: take the amount/number of working day => OK :)
# 
# PLANNING FOR  6 MONTH...
# 
# select date_trunc ('week', series.date)::date as week,date_trunc ('month', series.date)::date as month,series.date,l.amount/working_day_in_dates(min(l.date_from),max(l.date_to)) as amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id
# from (
#		 select non_weekends_between(current_date,(current_date+ interval '6 month')::date
#			 /*select date_from,date_to from report_account_analytic_planning_line*/
#			 ) as date
#	 ) as series
#	 /*select lines that cross the date, like in advanced planning line search wizard*/
# left outer join report_account_analytic_planning_line l on (series.date >= l.date_from and series.date <= l.date_to) group by series.date,l.amount,l.planning_id,l.manager,l.account_id,l.department_id,l.user_id,l.product_id,l.activity_id
# order by series.date;
# 
# 
# 
# -------------------
# -------------------
# postgres=# select date, count(*) from sales group by date order by date;
#	 date	| count
# ------------+-------
#  2008-05-05 |	 2
#  2008-05-06 |	 2
#  2008-05-08 |	 3
#  2008-05-09 |	 2
# (4 rows)
# This basically works, but it hides one important fact — no sales happened on May 7.
# 
# To fix this, we can use generate_series. When you run this function normally, it just generates a series of numbers:
# 
# postgres=# select generate_series(0,4);
# generate_series
# -----------------
# 0
# 1
# 2
# 3
# 4
# (5 rows)
# However, you can easily change it to generate a series of dates:
# 
# postgres=# select generate_series(0,4) + date '2008-05-05' as date;
# 
# For us:
# 
# select generate_series(0,Number of working day between start and end date) + 
# 
# 
# date
# ------------
# 2008-05-05
# 2008-05-06
# 2008-05-07
# 2008-05-08
# 2008-05-09
# (5 rows)
# 
# select series.date, count(sales.date)
# from (select generate_series(0,4) + date '2008-05-05' as date) as series
# left outer join sales on series.date=sales.date group by series.date
# order by series.date;
# 
#	 date	| count
# ------------+-------
#  2008-05-05 |	 2
#  2008-05-06 |	 2
#  2008-05-07 |	 0
#  2008-05-08 |	 3
#  2008-05-09 |	 2
# (5 rows)