# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) Camptocamp SA
# Author: Arnaud Wüst
#
#
#    This file is part of the c2c_planning_management module
#    it links hr_holidays with c2c_planning_management: to create a line in 
#
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from osv import fields
from osv import osv
from datetime import datetime, timedelta
import time

from workload_tools import *


class hr_holidays_status(osv.osv):
    _inherit = "hr.holidays.status"
    _columns = {
        'activity' : fields.many2one('project.activity_al', 'Activity')
    }
hr_holidays_status()


class planning_hr_holidays(osv.osv):
    
    _inherit = 'hr.holidays'
    _columns = {
        'planning_line_id' : fields.many2one('report_account_analytic.planning.line', 'Planning Line', invisible=True, readonly=True, relate=True),
    }
    
    def _get_account_id(self, cr, uid, holiday_obj, context={}):
        """return the account's id to use in the planning line"""
        if holiday_obj.holiday_status.analytic_account.id:
            return holiday_obj.holiday_status.analytic_account.id
        return False

    
    def add_planning_lines(self, cr, uid, ids):
        """create the planning lines associated with the holiday request """
        errorMessage = ""
        
                
        #find the default planning
        planning_obj = self.pool.get('report_account_analytic.planning')
        default_planning_id = planning_obj._get_default_planning(cr, uid)        
        if default_planning_id == False:
            errorMessage = "No default planning defined"
  
            
        #get manager and amount unit
        if errorMessage == "":
            manager_id = uid
            default_planning = planning_obj.browse(cr, uid, default_planning_id)
            amount_unit = default_planning.default_uom

              

        holidays = self.browse(cr, uid, ids)        
        for h in holidays:          

            values = {}
            if errorMessage == "":            
                values['planning_id'] = default_planning_id
                values['manager'] = manager_id
                values['amount_unit'] = amount_unit.id
                
                values['holiday_request'] = h.id
                
            #find the account linked to holidays 
            if errorMessage == "":    
                account = self._get_account_id(cr, uid, h)        
                if account:
                    values['account_id'] = account
                else:
                    errorMessage = "No analytic account linked to holiday status: " + h.holiday_status.name+" (id= "+ str(h.holiday_status.id) +")" 


            # get employee and user    
            if errorMessage == "":
                #Is the employee defined in the holiday? 
                if h.employee_id:
                    employee = h.employee_id
                    
                    #find the user related to employee
                    if h.employee_id.user_id:
                        user = h.employee_id.user_id
                    else: 
                        errorMessage = "No user defined for employee: %s (id= %s) " % (h.employee_id.name, str(h.employee_id.id))
                        
                # if the employee is not defined in holiday, we maybe can user the user
                elif h.user_id: 
                    user = h.user_id
                    
                    #find the employee linked to the user
                    employee_obj = self.pool.get('hr.employee')
                    employee_ids = employee_obj.search(cr, uid, [('user_id', '=', h.user_id.id )])
                    if employee_ids and len(employee_ids) > 0:
                        employee = employee_obj.browse(cr, uid, employee_ids[0])
                    else:
                        errorMessage = "No employee defined for user: %s (id= %s) " % (h.user_id.name, str(h.user_id.id))
                else: 
                    errorMessage = "No user defined for holiday request: %s (id= %s) " % (h.name, str(h.id))
                    
                                        
            #store the user_id
            if errorMessage == "":            
                values['user_id'] = user.id
                    
                    
            #find the departement
            if errorMessage == "":            
                query = """SELECT department_id 
                    FROM hr_department_user_rel 
                    WHERE user_id=%d ORDER BY department_id"""% user.id
                cr.execute(query)
                res = cr.dictfetchall()     
                # define the department if there is one and only one department for the user. If there is more than one, leave it empty
                if len(res) == 1:
                    values['department_id']=res[0]['department_id']
                    
                
            #find the product linked to the employee
            if errorMessage == "":
                if employee.product_id: 
                    values['product_id'] = employee.product_id.id
                else: 
                    errorMessage = "No product defined for employee: " + employee.name + " (id= "+ str(employee.id) +")"
          
                
            #get the activity
            if errorMessage == "":
                if h.holiday_status.activity: 
                    values['activity_id'] = h.holiday_status.activity.id
                else: 
                    errorMessage = "No activity defined for holiday status: " + h.holiday_status.name + " (id= "+ str(h.holiday_status.id) +")"
          
                    
            #get date_from, date_to, amount
            if errorMessage == "":
                datetime_from = datetime(*(time.strptime(h.date_from, '%Y-%m-%d %H:%M:%S')[0:6]))
                datetime_to   = datetime(*(time.strptime(h.date_to,   '%Y-%m-%d %H:%M:%S')[0:6]))
                
                values['date_from'] = datetime_from.strftime("%Y-%m-%d")
                values['date_to']   = datetime_to.strftime("%Y-%m-%d")

                my_user = User(h.user_id, datetime_from, datetime_to)
                values['amount'] = my_user.get_working_time(cr, uid, Period(datetime_from, datetime_to))
                
                values['note'] = h.name+' Holidays Request'
                
            
            #we have all informations, let's create the planning lines
            if errorMessage == "":
                
                id = self.pool.get('report_account_analytic.planning.line').create(cr, uid, values, {})
                
                #and let's link this holiday request to the planning id
                self.write(cr, uid, h.id, {'planning_line_id': id})
            
        if errorMessage != "":
            raise osv.except_osv('Unable to create related planning lines.', errorMessage)
            #self.logger.notifyChannel("warning", netsvc.LOG_WARNING, errorMessage)    
        else:   
            return True         
        
        
    def remove_planning_lines(self, cr, uid, ids):
        """delete the planning lines associated with the holiday request """
        
        #fine lines associated with these holidays
        holidays = self.browse(cr, uid, ids)
        
        lines_ids = []
        for h in holidays:
            if h.planning_line_id:
                lines_ids.append(h.planning_line_id.id)

        #delete lines
        self.pool.get('report_account_analytic.planning.line').straight_unlink(cr, uid, lines_ids) 

        #set the the planning line id to none
        self.write(cr, uid, ids,  {'planning_line_id':None})            
                

    def holidays_refuse(self, cr, uid, ids, *args):
        """called when the user press the button refuse"""
        
        self.remove_planning_lines(cr, uid, ids)
        return super(planning_hr_holidays, self).holidays_refuse(cr, uid, ids, args)
        

    def holidays_cancel(self, cr, uid, ids, *args):
        """called when theuser press the button cancel"""
        
        self.remove_planning_lines(cr, uid, ids)        
        return super(planning_hr_holidays, self).holidays_cancel(cr, uid, ids, args)

        
    def holidays_validate(self, cr, uid, ids, *args):
        """called when theuser press the button validate"""
        
        self.add_planning_lines(cr, uid, ids)       
        return super(planning_hr_holidays, self).holidays_validate(cr, uid, ids, args)



#    def set_to_draft(self, cr, uid, ids, *args):
#        print "SET TO DRAFT"       
#        return super(planning_hr_holidays, self).set_to_draft(cr, uid, ids, args)
#    
#    def holidays_confirm(self, cr, uid, ids, *args):
#        print "holidays confirm"
#        return super(planning_hr_holidays, self).holidays_confirm(cr, uid, ids, args)    
#
#    def holidays_draft(self, cr, uid, ids, *args):
#        print "holidays draft"
#        return super(planning_hr_holidays, self).holidays_draft(cr, uid, ids, args)

planning_hr_holidays()