# -*- coding: utf-8 -*- 
#!/usr/bin/env python2.3
#
#  planning_by_function.py
#  planning
#
#  Created by Nicolas Bessi on 19.08.08.
#  Copyright (c) 2008 CamptoCamp. All rights reserved.
#
from c2c_reporting_tools.translation import _

from report.interface import report_int
from c2c_reporting_tools.flowables.timebox_chart import  *
import pooler
import planning_by_type
from datetime import datetime, timedelta
import time

class Planning_by_function(planning_by_type.Planning_by_type):
                
    report_name = "Planning by function"
    
    def build_chart_datas(self):
        """ retrieve and format the planning lines to a structure that can be used to draw the chart """
            
        periods = {}
        
        planning_names = {}
        phase_names = {}
        function_names = {}
        
       
        #parse all lines and sort them in a tree:  planning _id-> project_id -> function_id ->periods_list
        lines = pooler.get_pool(self.cr.dbname).get('report_account_analytic.planning.line').browse(self.cr, self.uid, self.ids)
        for line in lines:            
            #does the planning dictionnary already exists in the tree. If not, we must init it
            if line.planning_id.id not in periods:
                periods[line.planning_id.id] = {}
            #does the project dictionnary alrady exists in the tree. If not, we must init it
            if line.account_id.id not in periods[line.planning_id.id]:
                periods[line.planning_id.id][line.account_id.id] = {}
            #does the function already exists in the tree. If not, we must init it 
            if line.product_id.id not in periods[line.planning_id.id][line.account_id.id]:
                periods[line.planning_id.id][line.account_id.id][line.product_id.id] = []
                    
            #store the period of the line in the planning/tree    
            periods[line.planning_id.id][line.account_id.id][line.product_id.id].append((time.strptime(line.date_from, "%Y-%m-%d"), time.strptime(line.date_to, "%Y-%m-%d"), str(int(line.unified_amount)), line.user_id.name))
            
            #also store the names of the elements
            planning_names[line.planning_id.id] = line.planning_id.name
            phase_names[line.account_id.id] = line.account_id.name_get(self.cr, self.uid, [line.account_id.id])[0][1] #use .name_get() instead of .name to get the full name with parent's names
            function_names[line.product_id.id] = line.product_id.name
                    
        #no more need of the var lines
        del lines
        
        
        charts_datas = {}

        date_start, date_end = self.get_form_boundary_dates()
        
        #now the line are sorted, we can build the data
        for planning_id in periods:
            
            
            
            #one chart by planning
            charts_datas[planning_id] = TimeBoxesChartLegendBuilder("", _("Users:"), date_start, date_end)            
            
            
            for phase_id in periods[planning_id]:
                #each phase is a title 
                charts_datas[planning_id].append_Y_item('<b>'+phase_names[phase_id]+'</b>')                
                
                for function_id in periods[planning_id][phase_id]:
                    #each function is an item of the chart
                    item_id = str(phase_id)+" "+str(function_id)
                    
                    charts_datas[planning_id].append_Y_item(function_names[function_id], 1, item_id)
                        
                    cpt = 0   
                    for period in periods[planning_id][phase_id][function_id]:
                        #each period is a box in the chart
                        period_date_start = period[0]
                        period_date_end = period[1]
                        period_time= period[2]
                        period_text = period[3]
                        
                        charts_datas[planning_id].add_period(item_id, period_date_start, period_date_end,None, period_time+"h - "+period_text,period_text)
            
            #retrieve boundary dates to put them in the title
            (boundary_date_start, boundary_date_end) = charts_datas[planning_id].get_boundary_dates()
            #add dates to the chart title
            title = planning_names[planning_id]+" from "+boundary_date_start.strftime("%d.%m.%Y")+" to "+boundary_date_end.strftime("%d.%m.%Y")
            charts_datas[planning_id].set_title(title)

        return charts_datas
        
Planning_by_function('report.planning_by_function')