# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) Camptocamp SA
# Author: Arnaud Wüst
#
#
#    This file is part of the c2c_planning_management module
#
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from c2c_reporting_tools.translation import _

from report.interface import report_int
from c2c_reporting_tools.templates.standard_template import *
from reportlab.platypus import PageBreak
import pooler
from datetime import datetime, timedelta
import time
import netsvc


class Planning_by_type(report_int):
    """ the open erp object that represent the rapport. It contain the logic to collect the data to build the rapport 
        This class is the parent of all reports It provide commons methods.
        To use it, inherit from this object and overload build_chart_datas() method 
    """
    
    report_name = "Graphical planning" #should be redefined in subclasses
    
    uid = None
    ids = None
    cr = None
    datas = None
    context = None

    logger = netsvc.Logger()
    LOG_NAME = 'report'       


    def get_planning_ids(self):
        """ return the list of planning ids to display """
        
        planning_ids =  []
        #if there is some planning lines selected
        if len(self.datas['form']['ids']) :
        
            query = """ SELECT distinct(planning_id) 
                        FROM report_account_analytic_planning_line 
                        WHERE id IN ( %s) """ %(','.join(map(str,self.datas['form']['ids'])))
            self.cr.execute(query)
            planning_ids = map(lambda x:x[0], self.cr.fetchall())
        
        return planning_ids


    
    def get_form_boundary_dates(self):
        """ for a given planning, return the start and end date of the scale if defined in the form"""
        date_start = None
        #do we come from the wizard?
        if 'form' in self.datas and 'date_start' in self.datas['form']:         
            #start date comes from the start date limit defined in the wizard
            date_start = time.strptime(self.datas['form']['date_start'], "%Y-%m-%d")
        
        #end date comes from the end date limit defined in the wizard if defined
        date_end = None
        if 'form' in self.datas and 'date_end' in self.datas['form'] and self.datas['form']['date_end'] != False:
            date_end = time.strptime(self.datas['form']['date_end'], "%Y-%m-%d")
                
        return (date_start, date_end)
                
                
    
    def build_chart_datas(self):
        """ retrieve and format the planning lines to a structure that can be used to draw the chart 
            this method should be overloaded in the subclass to return the right datas
        """
            
        raise Exception("You must overload the method 'build_chart_datas' to return an object 'TimeBoxesChartData'.")
        return{}
    
    
            
    def create(self, cr, uid, ids, datas, context={}):
        """ construct the report """

        #init
        self.cr = cr
        self.uid= uid
        self.datas = datas
        
        #lines ids come from the wizard form
        if 'form' in datas and 'ids' in datas['form']:
            self.ids = self.datas['form']['ids']
        #lines ids come from the line selection
        else:
            self.ids = ids
            
        self.context = context
        self.pool = pooler.get_pool(self.cr.dbname)

        
        
        doc = STLandscapeTotalPage()
        user = self.pool.get('res.users').browse(cr,uid,uid, context)        
        doc.company_name = user.company_id.name
        doc.report_name = self.report_name
               
        story = []
    
    
        #retrieve the planning_lines informations in a format ready to build the charts
#        try: 
        chart_builders = self.build_chart_datas()

        # foreach planning build a chart
        for planning_id in chart_builders:
            
            #add it to the story
            story.append(chart_builders[planning_id].get_flowable(doc.get_content_width()))
            story.append(chart_builders[planning_id].get_legend_flowable(doc.get_content_width()))
            story.append(PageBreak())
            

            
        
        #no more need of chart_datas
        del chart_builders
            
        return doc.fillWith(story)
#        except Exception, e:
#            self.logger.notifyChannel(self.LOG_NAME, netsvc.LOG_WARNING, "Report Error: %s " % e)
#            return doc.fillWith([])


Planning_by_type('report.planning_by_type')