# -*- coding: utf-8 -*- 
#!/usr/bin/env python2.3
#
#  Planning_by_user.py
#  planning
#
#  Created by Nicolas Bessi on 19.08.08.
#  Copyright (c) 2008 CamptoCamp. All rights reserved.
#

from c2c_reporting_tools.translation import _

from report.interface import report_int
from c2c_reporting_tools.flowables.timebox_chart import  *
import pooler
import planning_by_type
from numpy import outer
import Numeric
import time
import osv


#define shell classes

class Planning :
    'shell class used for representing planning'
    def __init__(self, browseRec):
        'Constructor that take a planning browse record'
        self.br = browseRec
        self.projects = []
        
    def __cmp__(self, other):
        'comparator function'
        return cmp(self.br.name, other.br.name) 
    
    def compare(self, planning) :
        'filter function'
        if not self.br or not planning:
            return False
        if self.br.id == planning.id :
            return True
        else :
            return False

        
class Project :
    "projet shell class"
    
    def __init__(self, browseRec):
        'Constructor that take a Project browse record'
        self.br = browseRec
        self.users = []
        
    def __cmp__(self, other):
        'comparator function'
        return cmp(self.br.name_get(self.br._cr,self.br._uid, [self.br.id])[0][1],
            other.br.name_get(other.br._cr,other.br._uid, [other.br.id])[0][1]) 
        
    def compare(self, project) :
        'filter function'
        if not self.br or not project:
            return False
        if self.br.id == project.id :
            return True
        else :
            return False
    
class User :
    "User shell class"
    
    def __init__(self, browseRec):
        'Constructor that take a User browse record'
        self.br = browseRec
        # we do this because user is not requierd on planning line
        if type(browseRec) == osv.orm.browse_null:
            # if we call it like taht the user should be at the end of the sort
            self.name = 'zzz'
        else :
            self.name = self.br.name 
        self.periods = []
        
    def __cmp__(self, other):
        'comparator function'
        return cmp(self.name, other.name) 
        
    def compare(self, user) :
        if type(user) == osv.orm.browse_null:
            if self.name == 'zzz':
                return True
            else :
                return False
        if self.name == user.name :
            return True
        else :
            return False

class Period :
    """Shell class fully forged does not receved a browse record 
    It map the period item of the chart"""
    def __init__(self, start, end, planning_id, text, group_text):
        'Constructor'
        self.start = start
        self.end = end
        self.planning_id = planning_id
        self.text = text
        self.group_text = group_text


class Planning_by_user(planning_by_type.Planning_by_type):
    """ Show the user ressources booking grouped by project, user"""
                    
    report_name = "Planning by user"
        
    def build_chart_datas(self):
        """ retrieve and format the planning lines to a structure that can be used to draw the chart """
        try :
            # contain all of the planning, it's a kind of root node
            plannings = []
           
            # parse all lines and sort them in a object hirearchy:  planning->project->users->period 
            lines = pooler.get_pool(self.cr.dbname).get('report_account_analytic.planning.line').browse(self.cr, self.uid, self.ids)
            # line ids are given by planning-by_type.py
            for line in lines:
                # for ach line we see if the planning exist in the plannings list
                active_planning = [pl for pl in plannings if pl.compare(line.planning_id)]
                # if active planning is not empty
                if active_planning :
                    active_planning = active_planning[0]
                else :
                    # we add the planning to the planning list
                    active_planning = Planning(line.planning_id)
                    plannings.append(active_planning)
                # we look if the project is in the active planning
                active_project = [p for p in active_planning.projects if p.compare(line.account_id)]
                # if yes we get it 
                if  active_project :
                    active_project = active_project[0]
                # we add it to the planning
                else :
                    active_project = Project(line.account_id)
                    active_planning.projects.append(active_project)
                active_user = [u for u in active_project.users if u.compare(line.user_id)]
                # same as before but for the user, the user is linked to the project
                if  active_user :
                    active_user = active_user[0]
                else :
                    active_user = User(line.user_id)
                    active_project.users.append(active_user)
                # we create a temporary period that is linked to the user
                tmp_period = Period(time.strptime(line.date_from, "%Y-%m-%d"),
                    time.strptime(line.date_to, "%Y-%m-%d"), 
                    line.planning_id.id, 
                    str(int(line.unified_amount))+"h - "+line.activity_id.name,
                    line.activity_id.name)
                active_user.periods.append(tmp_period)

            charts_datas = {}
            #we retrieves the planning boundaries
            date_start, date_end = self.get_form_boundary_dates()
            # we sort the planning it use the __comp__ function
            plannings.sort()

            # we built the chart data
            for plan in plannings :
                charts_datas[plan.br.id] = TimeBoxesChartLegendBuilder("", _("Activities:"), date_start, date_end)
                plan.projects.sort()
                for proj in plan.projects:
                    charts_datas[plan.br.id].append_Y_item('<b>'+proj.br.name_get(self.cr, self.uid, [proj.br.id])[0][1]+'</b>', 1)
                    proj.users.sort()
                    for user in proj.users :
                        #we group all the unattributed lines
                        if user.name == 'zzz':
                            user.itemid = charts_datas[plan.br.id].append_Y_item('<para textColor="red">No user attributed</para>', 2)
                        else :
                             user.itemid = charts_datas[plan.br.id].append_Y_item(user.name, 2)
                        for prd in user.periods :
                            charts_datas[plan.br.id].add_period(user.itemid, prd.start, prd.end, None, prd.text, prd.group_text)
                            
                #retrieve boundary dates to put them in the title
                (boundary_date_start, boundary_date_end) = charts_datas[plan.br.id].get_boundary_dates()
                #add dates to the chart title
                title = plan.br.name+" from "+boundary_date_start.strftime("%d.%m.%Y")+" to "+boundary_date_end.strftime("%d.%m.%Y")
                charts_datas[plan.br.id].set_title(title)
                    
        except Exception, e:
            raise e
        return charts_datas        
Planning_by_user('report.planning_by_proj_user')