# -*- coding: iso-8859-1 -*-
##############################################################################
#
# Copyright (c) 2008 Camptocamp SA (http://www.camptocamp.com) All Rights Reserved.
# @Author: Aubort Jean-Baptiste
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import wizard
import tools
import base64
import StringIO
import csv
import pooler
import safe_load

class wizard_synchro_lang(wizard.interface):
    # Get available language codes
    def _get_language(self, cr, uid, context):
        lang_obj=pooler.get_pool(cr.dbname).get('res.lang')
        ids=lang_obj.search(cr, uid, [('active', '=', True),])
        langs=lang_obj.browse(cr, uid, ids)
        return [(lang.code, lang.translatable and lang.name or 'New language') for lang in langs]

    # Get language name from the code
    def _get_lang_name(self, cr, uid, context, code):
        lang_obj=pooler.get_pool(cr.dbname).get('res.lang')
        ids=lang_obj.search(cr, uid, [('code', '=', code),])
        lang=lang_obj.browse(cr, uid, ids)
        return len(lang)>0 and lang[0].code or ''

    # Generate translation, import them in database (safe by default)
    def _sync_trans(self, cr, uid, data, context):
        form = data['form']
        # Get translation
        file=tools.trans_generate(form['lang'], 'all', dbname=cr.dbname)
        buf=StringIO.StringIO()
        writer=csv.writer(buf, 'TINY')
        cpt_row=0
        for row in file:
            cpt_row+=1
            # If there is no translation value, set it like the source (so it shows up in "All terms" menu)
            if cpt_row!=0 and row[4]=='':
                row[4]=row[3]
            writer.writerow(row)
        del file
        lang_name = self._get_lang_name(cr, uid, data, form['lang'])
        data = buf.getvalue().split('\n')
        # Load translations in database
        safe_load.trans_load_data_safe(cr.dbname, data, form['lang'], True, lang_name, form['safe'])
        buf.close()
        return {}

    fields_form={
        'lang': {'string':'Language', 'type':'selection', 'selection':_get_language,},
        'safe': {'string':'Safe', 'type':'boolean', 'default': True},
    }

    view_form_init="""<?xml version="1.0"?>
    <form string="Synchronize language">
        <image name="gtk-dialog-info" colspan="2"/>
        <group colspan="2" col="4">
            <label align="0.0" string="Choose a language to synchronize:" colspan="4"/>
            <field name="lang" colspan="4"/>
            <field name="safe" colspan="4"/>
            <label string="Tick the Safe checkbox to preserve existing translations." colspan="4" align="0.0"/>
        </group>
    </form>"""

    view_form_finish="""<?xml version="1.0"?>
    <form string="Synchronize language">
        <image name="gtk-dialog-info" colspan="2"/>
        <group colspan="2" col="4">
            <label align="0.0" string="Synchronization done succefully !" colspan="4"/>
        </group>
    </form>"""

    states={
        'init':{
            'actions': [],
            'result': {'type': 'form', 'arch': view_form_init, 'fields': fields_form, 'state': [('end', 'Cancel', 'gtk-cancel'),('finish', 'Ok', 'gtk-ok', True)]}
        },
        'finish':{
            'actions': [_sync_trans],
            'result': {'type': 'form', 'arch': view_form_finish, 'state': [('end', 'Ok', 'gtk-ok', True)]}
        },
    }
wizard_synchro_lang('module.lang.synchro')
