# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2008 Sylëam Info Services (http://www.syleam.fr) All rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import time
from osv import fields, osv

#
# List of configuration section
#
class res_config_section(osv.osv):
    _name = 'res.config.section'
    _description = 'Configuration section'

    _columns = {
        'name': fields.char('Name', size=16, required=True),
        'description': fields.char('Description', size=64, translate=True),
    }

    # indique l'ordre de tri par défaut
    _order = 'name'

    _sql_constraints = [
        ('name','UNIQUE (name)' , 'the name must be unique !')
    ]
    

res_config_section()

#
# List of keyword per section
#
class res_config(osv.osv):
    _name = 'res.config'
    _description = 'Keyword configuration per section'

    _columns = {
        'name': fields.char('Name', size=32, required=True),
        'description': fields.char('Description', size=256, translate=True),
        'value': fields.char('Value', size=64),
        'section_id': fields.many2one('res.config.section','Section',required=True),
        'active': fields.boolean('Active'),
    }

    _defaults = {
        'active': lambda *a: True,
    }

    # search and return the value for keyword and section
    def _get(self, cr, uid, section, keyword):
        # Search section first
        section_id = self.pool.get('res.config.section').search(cr, uid, [('name','=', section)])
        if not section_id:
            print "WARNING res.config : section " + str(section) + " not found"
            return False
        keyword_args = [('name','=',keyword),
                        ('active','=',True),
                        ('section_id','=',section_id[0])]
        keyword_id = self.search(cr, uid, keyword_args)
        if not keyword_id:
            print "WARNING res.config : section " + str(section) + " -> keyword " + str(keyword) + " not found"
            return False
        keyword_value = self.read(cr, uid, keyword_id, ['value'])
        return keyword_value[0]['value']

res_config()

