##############################################################################
#
# Copyright (c) 2005-2008 TINY SPRL. (http://tiny.be) All Rights Reserved.
#
# $Id$
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import wizard
#import netsvc
import pooler
#import math
import mx.DateTime
from mx.DateTime import RelativeDateTime

from osv.osv import except_osv

_form = """<?xml version="1.0"?>
 <form string="Automatic Planification">
	<field name="product_id"/>
	<field name="date_start"/>
	<field name="product_qty"/>
	<field name="product_uom"/>
    <separator colspan="4" string=""/>
	<field name="monday"/><newline/>
	<field name="tuesday"/><newline/>
	<field name="wednesday"/><newline/>
	<field name="thursday"/><newline/>
	<field name="friday"/><newline/>
	<field name="saturday"/><newline/>
	<field name="sunday"/><newline/>
 </form>
"""

_fields = {
	'product_id': {'string':'Product', 'type':'many2one', 'relation':'product.product', 'required':True},
	'product_qty':{'string':'Quantity', 'type':'float', 'digits':(16,2), 'require':True},
	'product_uom':{'string':'UOM', 'type':'many2one', 'relation':'product.uom', 'required':True},
    'date_start': {'string':'Start Date', 'type':'date', 'required':True},
}
_days = ('monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday')
for d in _days:
	_fields[d] = {'string': d.capitalize(), 'type': 'float', 'digits':(16,2)}


class automatic_planification(wizard.interface):
    def run(self, cr, uid, datas, context=None):
		so = pooler.get_pool(cr.dbname).get('sale.order').browse(cr, uid, datas['id'])
		form = datas['form']

		if not so.order_line or form['product_id'] not in [l.product_id.id for l in so.order_line]:
			raise except_osv(_('This product is not in the sales order lines'))
		
		qty = form['product_qty']
		if not qty:
			raise except_osv(_('You must set a quantity'))

		cpt = reduce(lambda acc, day: acc + float(form[day]), _days, 0)
		if not cpt:
			raise except_osv(_('You must set at least one day'))

		deliveryline = pooler.get_pool(cr.dbname).get('sale.delivery.line')
		day = mx.DateTime.strptime(form['date_start'], '%Y-%m-%d')
		while qty > 0:
			to_deliver = min(qty, form[_days[day.day_of_week]])
			if to_deliver:
				deliveryline.create(cr, uid, {
					'order_id': so.id,
					'product_id': form['product_id'],
					'product_qty': to_deliver,
					'product_uom': form['product_uom'],
					'date_planned': day.strftime('%Y-%m-%d'),
				})
				qty -= to_deliver
			day = day + RelativeDateTime(days=1)

		return {}

        
    states = {
        'init' : {
            'actions' : [],
            'result' : {'type' : 'form',
                    'arch' : _form,
                    'fields' : _fields,
                    'state' : [('end', 'Cancel'),('run', 'Create delivery lines')]}
        },
        'run':{
            'actions' : [],
            'result' : {'type' : 'action',
                    'action' : run,
                    'state' : 'end'}
        }
	}

automatic_planification("sale.delivery.automatic_planification")

